﻿#Name: CommonFunctions.psm1
#Version: V02.08.00 - lower number used for internal release, next higher for MS release, top digit for major version change
#Release date/time: 2015.09.17
#Description: PAM install scripts common functions

#TLG ref: N/A

# prerequisites

#
# Revision history
# 2.6.01 15-Sep-2105 - Fixed Test-PasswordComplexity function
# (ocg\mike)
# 2.2.05 13-Sep-2014 - Fixed error level with test-domainjoin
# (ocg\burtt)
# 2.2.04 10-Sep-2015 - Changed scope of $OurRegistryLocation and $RestartRegistryName to global
# (ocg\mike) 
# 2.2.03 10-Sep-2015 - Added logging in numerous places
#  (ocg\mike)        - Corrected prompts for accounts
# 2.0.02 09-Sep-2015 - skipped this version as it had to be rolled back.

#BinilK getOrCreate was renamed to just Get-. The CreateIfMissing flag clears out any doubts


################################
# SCRIPT FUNCTIONS
################################
Function Get-LogFileName
{
    # log file name returned in format [$path\]filename_20150603T120101.log
    param(
        [Parameter(Mandatory=$true)] [string]$fileName, 
        [Parameter(Mandatory=$false)] [string]$path)
    # TODO: should we validate the path exists?
    if (!$path.EndsWith("\"))
    {
        $path = $path + "\"
    }
    $d = (Get-Date).ToString("yyyyMMddThhmmss")
    return $path + $filename+ "_" + $d + ".log"
}

Function Register-EventLogSource
{
    Param(
        [Parameter(Mandatory=$true)] [string]$eventLogName, 
        [Parameter(Mandatory=$true)] [string]$eventLogSource)

    if ([System.Diagnostics.EventLog]::Exists($eventLogName) -eq $true)
    {
        # Event log exists now see if source exists
        $message = [string]::format("Event log {0} exists", $eventLogName)
        write-host -fore green $message
        if ([System.Diagnostics.EventLog]::SourceExists($eventLogSource) -eq $true)
        {
            #event log source exists
            $message = [string]::format("Event log source {0} already registered", $eventLogSource)
            write-host -fore green $message
        }
        else
        {
            # event log source does not exist, so create
            $error.clear()
            $res = New-EventLog -LogName $eventLogName -Source $eventLogSource -ErrorAction SilentlyContinue
            if ($error.count -gt 0)
            {
                $message = [string]::format("Error registering  event log source {0}. Exception details: {1}", $eventLogSource, $error[0].Exception.Message)
                write-host -fore red $message
            }
        }
    }
    else
    {
        Write-Host -fore red "ERROR! Event Log $eventLogName does not exist."
    }
}

Function Write-LogsAndDisplay
{
    Param(
    [Parameter(Mandatory=$true)] [Int32]$EventID, 
    [Parameter(Mandatory=$true)] [String]$message,
    [Parameter(Mandatory=$false)] [ValidateSet('Error','Information','Warning')][string]$ErrorLevel='Information',
    [Parameter(Mandatory=$false)] [bool]$SuppressConsole = $false
    )
    #Validate Input
    # ErrorLevel must be Error, Information, or Warning
    # 
    if (($ErrorLevel.CompareTo('Error') -ne 0) -and ($ErrorLevel.CompareTo('Information') -ne 0) -and ($ErrorLevel.CompareTo('Warning') -ne 0))
    {
        write-host -fore yellow Supplied ErrorLevel value $ErrorLevel is not valid. Using Error instead
        $ErrorLevel = 'Warning'
    }

    [string]$forecolor = [string]::Empty
    if ($ErrorLevel.CompareTo('Error') -eq 0){$foreColor = 'red'}
    if ($ErrorLevel.CompareTo('Information') -eq 0){$foreColor = 'green'}
    if ($ErrorLevel.CompareTo('Warning') -eq 0){$foreColor = 'yellow'}
    $CallerDetails = [string]::Format(" @{0},{1}", $MyInvocation.ScriptName, $MyInvocation.ScriptLineNumber)
    if ( ($ErrorLevel -eq 'Error') -or ($ErrorLevel -eq 'Warning') )
    {
        $message = $message + $CallerDetails
    }
    if($suppressConsole -eq $false)
    {
        #Write-Host -fore $forecolor $ErrorLevel $message
        Write-Host -fore $forecolor $message
    }
    Write-LogFile -logFileName $global:LogFileName -ErrorLevel $ErrorLevel -message $message
    Write-EventLog -LogName $global:eventLogName -Source $global:eventLogSource -EntryType $ErrorLevel -EventID 11004 -Category 0 -Message $message
    # return $false
}

Function Write-LogFile
{
    param(
        [Parameter(Mandatory=$true)] [string]$logFileName, 
        [Parameter(Mandatory=$true)] [ValidateSet('Error','Information','Warning')][string]$ErrorLevel = 'Information',
        [Parameter(Mandatory=$true)] [string]$message
    )
    $log = new-object IO.StreamWriter($global:logFileName, $true)
    $d = (Get-Date).ToString("yyyy-MM-dd hh:mm:ss")
    # this will create a unique log file for each run
    $log.WriteLine([string]::format("{0}`t{1}`t{2}", $d, $ErrorLevel, $message))
    $log.Flush()
    $log.Close()
    $log.Dispose()
}

function Write-Divider
{
    $reply=Write-host -ForegroundColor  "--------------------------------------------"
}

Function Read-Data
{
    PARAM (
        [Parameter(Mandatory=$true)] [string]$text,
        [Parameter(Mandatory=$true)] [string]$title,
        [Parameter(Mandatory=$false)] [string]$default = "",
        [Parameter(Mandatory=$false)] [boolean]$MandatoryInput = $true

    )

    $displayTitle=$title

    if ($MandatoryInput)
    {
        $displayTitle = "$title - MANDATORY"
    }

    $data=""
    $dataValid = $false
    while ($dataValid -eq $false)
    {
        $data = [Microsoft.VisualBasic.Interaction]::InputBox("$text", "$displayTitle", "$default") 

        if ($MandatoryInput)
        {
            $dataValid = $data -ne ""

            if (-not $dataValid)
            {
                $displayTitle = "$title - MANDATORY!!"
            }

        }
        else
        {
            $dataValid = $true
        }
    }

    Return $data

}

Function Show-Progress ($process, $color, $interval, $msg)
{
    Write-Host  -ForegroundColor $color "Executing " + $msg + " "

    While (Get-Process -Name $process -ErrorAction SilentlyContinue)
    {
        Write-Host -ForegroundColor $color "." -NoNewline
        Start-Sleep $interval
    }
    Write-Host -ForegroundColor Green "Done."
   

}

function Read-HostWithDefault
{
    PARAM (
        [Parameter(Mandatory=$true)] [string]$msg,
        [Parameter(Mandatory=$true)] [string]$title,
        [Parameter(Mandatory=$true)] [string]$default,
        [Parameter(Mandatory=$true)] [boolean]$MandatoryInput

    )
        $answer=read-data -text $msg -title $title -default $default -MandatoryInput $MandatoryInput
        return $answer
}

 #Load ActiveDirectory if not present
Function Initialize-ActiveDirectoryModule
{
	if(@(get-module | where-object {$_.Name -eq "activedirectory"} ).count -eq 0) 
    {
        Import-Module ActiveDirectory -ErrorAction SilentlyContinue
    }
}

function Convert-SecurePasswordToBytes($securePassword)
{
    $bytes = ConvertFrom-SecureString $securePassword  

    $plainText=Convert-PasswordBytesToPlaintext($bytes)

    Return $bytes
}

function Convert-PasswordBytesToPlaintext($encPassword)
{
    if ($encPassword -eq $null -or $encPassword -eq "")
    {
        return $null
    }
    else
    {
        try
        {
            $password = $encPassword  | ConvertTo-securestring
            $plaintext = [Runtime.InteropServices.Marshal]::PtrToStringAuto([Runtime.InteropServices.Marshal]::SecureStringToBSTR($password))

            return $plaintext
        }
        catch
        {
            return $null
        }
    }
}

# AD account management
Function Confirm-LocalSession
{
    If ($Host.Name -eq "ServerRemoteHost") 
    {
        Return $false
    }
    Else 
    {
        Return $true
    }
}

function Write-Divider
{
    write-host "--------------------------------------------"
}

# ====================================================================================
# Func: Show-Progress
# Desc: Shows a row of dots to let us know that $process is still running
# ====================================================================================
Function Show-Progress ($process, $color, $interval, $msg)
{
    Write-Host  -ForegroundColor $color "Executing " + $msg + " "

    While (Get-Process -Name $process -ErrorAction SilentlyContinue)
    {
        Write-Host -ForegroundColor $color "." -NoNewline
        Start-Sleep $interval
    }
    Write-Host -ForegroundColor Green "Done."
   

}

#region Server Tests
Function Test-ServerOSVersion
{
	$cim = Get-CimInstance Win32_OperatingSystem
	$retval = $false
	$arrVer = $cim.version.Split(".")
	# we should have an array of at least three elements
        # the [0] element contains the "major" version number
	# the [1] element contains the "minor" version number
	# the [2] element contains the build number
	# NB: Server and Workstation cannot be differentiated by this numbering scheme.
	If ($arrVer.Length -gt 1)
	{
		If ($arrVer[0] -ge 6)
		{
		# we have Windows 2008 or later. This is minimum major version.

			If ($arrver[1] -ge 3)
			{
				# we have Windows 2012 R2 or later
				# 0 = Server 2008
				# 1 = Server 2008 R2
				# 2 = Server 2012
				# 3 = Server 2012 R2 -- this is our minimum minor version
				write-host -fore green Installed operating system is  $cim.Version
				$retval = $true
			}
			Else
			{
				write-host -fore red Privileged domain requires Microsoft Windows 2012 R2 or later
				write-host -fore red Installed operating system is $cim.Caption
                $message = [string]::Format("SQL can not be installed, Target server needs to be running Windows 2012 R2 or later. The server is running {0}", $cim.Caption)
                Write-EventLog -LogName Application -Source PAM-SQLInstall -EventId 25110 -EntryType Error -Message $message
				$retval = $false
			}
		}
	}
	return $retval
}

Function Test-IsWindowsServer
{
	$retval = $false
	# check operating system is Windows 2012 R2
	# Get-WmiObject has been deprecated. We should use Get-CimInstance instead.

	$cim = Get-CimInstance Win32_OperatingSystem
	# make sure running on server and not workstation (version numbers are same)
	If ($cim.Caption.IndexOf("Server") -eq 0)
	{
        $message =  [string]::Format("Windows Server 2012 R2 operating system is required. Installed operating system is {0}", $cim.Caption)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
    	$retval = $false;
	}
	else
	{
        $message = [string]::Format("Installed operating system is {0}", $cim.caption)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsVersion2012R2orHigher
{
    $cim = Get-CimInstance Win32_OperatingSystem
	$retval = $false
	$arrVer = $cim.version.Split(".")
	# we should have an array of at least three elements
    # the [0] element contains the "major" version number
	# the [1] element contains the "minor" version number
	# the [2] element contains the build number
	# NB: Server and Workstation cannot be differentiated by this numbering scheme.
	If ($arrVer.Length -gt 1)
	{
		If ($arrVer[0] -ge 6)
		{
		# we have Windows 2008 or later. This is minimum major version.

			If ($arrver[1] -ge 3)
			{
				# we have Windows 2012 R2 or later
				# 0 = Server 2008
				# 1 = Server 2008 R2
				# 2 = Server 2012
				# 3 = Server 2012 R2 -- this is our minimum minor version
                $message = [string]::Format("Installed operating system is {0}, version {1}.", $cim.Caption, $cim.Version)
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
				$retval = $true
			}
			Else
			{
                $message = [string]::Format("Privileged domain requires Microsoft Windows 2012 R2 or later. Installed operating system is {0}.", $cim.Caption)
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
				$retval = $false
			}
		}
	}
	return $retval
}

function Test-IsStaticIP
{
	$retval = $true
    $adap = Get-WmiObject win32_networkadapterconfiguration -Filter 'ipenabled = "true" and dhcpEnabled = "false"'
    if ($adap -eq $null)
    {
        # there are no matching adapters
        $message = [string]::format("Cannot find any IP-enabled interface with a static IP address.")
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
        return $false
    }
    else
    {
        # loop through the adapters. Ideally, we should only have one.
        # curiously, is $adap -eq $null, it has a .Count property value of 0. How does a null variable have any properties?
        # if $adap -ne $null and it has a single adapter, the .Count property is $null
        if ($adap.Count -eq $null)
        {
            $message = "Found a single adapter with a static IP address."
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
            $retval = $true
        }
        foreach ($a in $adap)
        {
            $message = "Adapter Details`n"
            $message += [string]::Format("Hostname: {0}`n", $a.DNSHostName)
            $message += [string]::Format("Description: {0}`n", $a.Description)
            $message += [string]::Format("MacAddress: {0}`n", $a.MacAddress)
            $message += [string]::Format("AdapterIndex: {0}`n", $a.Index)
            foreach($ip in $a.IPAddress)
            {
                if($ip.Contains("."))
                {
                    $message += [string]::format("IPv4 Address: {0}`n", $ip)
                }
                if ($ip.Contains(":"))
                {
                    $message += [string]::format("IPv6 Address: {0}`n", $ip)
                }
            }
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
        }
    }
    # do we need to test for 127.0.0.1 well-known loopback address?

	return $retval
}

Function Test-IsInDomain
{
    # TODO: Move the error and exit logic out of this function so it is handled by caller
	$retval = $false
	# check to see if computer is joined to a domain
	# TODO: convert this to use CIM namespace
	$wmi = (Get-WmiObject win32_computersystem)
	if ($wmi.partofdomain -eq $true)
	{
		$message = 'Machine is part of the existing domain. It cannot become the first domain controller is a new forest.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		exit
	}
	else
	{
		$message = [string]::format("This computer, {0}, is not part of a domain. It is ready to become the first domain controller in a new forest.", $env:COMPUTERNAME)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		return $true
	}
	return $retval
}

Function Get-NetBIOSName
{
    Param([Parameter(Mandatory=$true)][string]$DomainName)
	$NetBIOSName = $DomainName.Substring(0, $DomainName.IndexOf(".")).ToUpper()
	if ($NetBIOSName.length -gt 15)
	{
		$NetBIOSName = $NetBIOSName.Substring(0, 15)
        $message = [string]::Format("Left-most segment of {0} is longer than 15 characters. NetBIOSName cannot be longer than 15 characters. Trimming NetBIOSName to 15 characters: {1}.", $DomainName, $NetBIOSName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
	}
    elseif ($NetBIOSName.length -eq 0)
    {
        $message = [string]::Format("Did not successfully parse a NetBIOS name from {0}. Exiting...", $DomainName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
        Exit
    }
    else
	{
        $message = [string]::Format("NetBIOSName is {0}", $NetBIOSName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	}
	return $NetBIOSName
}
#endregion Server Tests

function Test-DomainJoin
{
	$retval = $false
	# check to see if computer is joined to a domain
	# TODO: convert this to use CIM namespace
	$wmi = (Get-WmiObject win32_computersystem)
	if ($wmi.partofdomain -eq $true)
	{
        $message = [string]::Format("Server is part of {0}", $wmi.Domain)
        Write-LogsAndDisplay -EventID 25101 -ErrorLevel Information -message $message
		write-host -fore green $message
		$retval = $true
	}
	else
	{
		write-host -fore red Machine is not part of a domain
		write-host -fore red This computer must be joined to a domain.
        $message = [string]::Format("SQL can not be installed on server {0} because it is not joined to a domain.", $wmi.Name)
        Write-EventLog -LogName Application -Source PAM-SQLInstall -EventId 25101 -EntryType Error -Message $message
		exit
	}
	return $retval
}

Function Test-PasswordComplexity
{
    Param(
        [Parameter(Mandatory=$true)] [SecureString]$Passwd
    )
    [boolean]$bReturn = $false;
    # requires retrieving secure-string from memory. Do not persist password in available. Work directly with BSTR.
    $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($Passwd)
    $ptPwd = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR)

    # $pwd = 'Pa$$w0rd'
    #password must be at least 15 characters
    $PasswordValidationTestsPassed = @()
    $PasswordValidationTestsFailed = @()

    $pwdLength = $ptPwd -match "(?=.{15,})"
    if ($pwdLength -eq $true)
    {
        $message = [string]::format("password is at least 15 characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password is less than 15 characters.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one lowercase character
    $pwdLowercase = $ptPwd -cmatch "[a-z]"
    if ($pwdLowercase -eq $true)
    {
        $message = [string]::format("password contains at least one lowercase characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one lowercase character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one UPPERCASE character
    $pwdUpperCase = $ptPwd -cmatch "[A-Z]"
    if ($pwdUpperCase -eq $true)
    {
        $message = [string]::format("password contains at least one UPPERCASE character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contain at least one UPPERCASE character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # password must contain at least one numeric or non-alphanumeric character
    $pwdNumericOrSpecial = $ptPwd -match "(?=.[\d\W])"
    if ($pwdNumericOrSpecial -eq $true)
    {
        $message = [string]::format("password contains at least one digit or special character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one digit or special character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # write summary
    if ($PasswordValidationTestsFailed.Length -gt 0)
    {
        write-host -fore red Password Failed Validation for the following reasons:
        for ( $n = 0; $n -lt $PasswordValidationTestsFailed.Length; $n++)
        {
            write-host -fore red $PasswordValidationTestsFailed[$n]
        }
    }
    else
    {
        write-host -fore green Password Passed Validation:
        for ( $n = 0; $n -lt $PasswordValidationTestsPassed.Length; $n++)
        {
            write-host -fore green $PasswordValidationTestsPassed[$n]
        }
        $bReturn = $true
    }
    return $bReturn
}

Function Test-AdminAccount {

    If (([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
    {    Param(
        [Parameter(Mandatory=$true)] [SecureString]$Passwd
    )
    $bReturn = $false;
    # requires retrieving secure-string from memory. Do not persist password in available. Work directly with BSTR.
    $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword)
    $ptPwd = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR)

    # $pwd = 'Pa$$w0rd'
    #password must be at least 15 characters
    $PasswordValidationTestsPassed = @()
    $PasswordValidationTestsFailed = @()

    $pwdLength = $ptPwd -match "(?=.{15,})"
    if ($pwdLength -eq $true)
    {
        $message = [string]::format("password is at least 15 characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password is less than 15 characters.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one lowercase character
    $pwdLowercase = $ptPwd -cmatch "[a-z]"
    if ($pwdLowercase -eq $true)
    {
        $message = [string]::format("password contains at least one lowercase characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one lowercase character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one UPPERCASE character
    $pwdUpperCase = $ptPwd -cmatch "[A-Z]"
    if ($pwdUpperCase -eq $true)
    {
        $message = [string]::format("password contains at least one UPPERCASE character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contain at least one UPPERCASE character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # password must contain at least one numeric or non-alphanumeric character
    $pwdNumericOrSpecial = $ptPwd -match "(?=.[\d\W])"
    if ($pwdNumericOrSpecial -eq $true)
    {
        $message = [string]::format("password contains at least one digit or special character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one digit or special character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # write summary
    if ($PasswordValidationTestsFailed.Length -gt 0)
    {
        write-host -fore red Password Failed Validation for the following reasons:
        for ( $n = 0; $n -lt $PasswordValidationTestsFailed.Length; $n++)
        {
            write-host -fore red $PasswordValidationTestsFailed[$n]
        }
    }
    else
    {
        write-host -fore green Password Passed Validation:
        for ( $n = 0; $n -lt $PasswordValidationTestsPassed.Length; $n++)
        {
            write-host -fore green $PasswordValidationTestsPassed[$n]
        }
        $bReturn = $true
    }
    return $bReturn

        Write-Host -fore Green "Script is running as Local Administrator."
    }
    ELSE
    {
        Write-Host -fore Red "Script is NOT running with Local Administrator privledges."
        Write-EventLog -LogName Application -Source PAM-SQLInstall -EventId 25140 -EntryType Error -Message "SQL can not be installed, script not running with Admin privledges."
        exit
    }

}

Function OurRegistryNode
{
     $path="HKLM:\SOFTWARE\$global:OurRegistryLocation\"
     $message = [string]::Format("Setting Registry Path to {0}", $path)
     Write-LogsAndDisplay -EventID 25051 -ErrorLevel Information -message $message
     if (-not (Test-Path -Path "$path"))
     {
        # $path does not exist so trying to create
         $message = [string]::Format("Registry Path {0} does not exist. Attempting to create....", $path)
        Write-LogsAndDisplay -EventID 25051 -ErrorLevel Information -message $message
        $ni = New-Item -Path "$path" -ErrorAction SilentlyContinue | Out-Null
        if ($ni -eq $null)
        {
            $message = [string]::Format("Failed to create Registry Path {0}", $path)
            Write-LogsAndDisplay -EventID 25051 -ErrorLevel Information -message $message
        }
        else
        {
            $message = [string]::Format("Successfully created Registry Path {0}", $path)
            Write-LogsAndDisplay -EventID 25051 -ErrorLevel Information -message $message
        }
     }

    return $path
    #return "HKLM:\SOFTWARE\$OurRegistryLocation\"
}

#region Restart Functions
Function IsRestartRequired
{
     $Path = OurRegistryNode 
     $RegistryNode=get-item -Path "$Path"
     if ($RegistryNode -eq $null)
     {
        return $False
     }

     $item = get-itemproperty -Path "$path" -ErrorAction SilentlyContinue
     return $item.$RestartRegistryName -eq 1
}

Function ClearRestartRequired
{
    $path="HKLM:\SOFTWARE\$global:OurRegistryLocation\"
    Remove-ItemProperty -Path "$path" -Name "$global:RestartRegistryName" -ErrorAction SilentlyContinue
}

Function Set-RestartRequired
{
    $Path = OurRegistryNode 
    $reply=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message  " - Setting Restart Required in the registry..."
    New-Item -Path "$Path" -ErrorAction SilentlyContinue | Out-Null
    $regKey = Get-Item -Path "$Path"
    $regKey | New-ItemProperty -Name "$global:RestartRegistryName" -PropertyType String -Value "1" -Force | Out-Null
}
#endregion Restart Functions

function InstallSettingFileName
{
    return "$DeployPAMFolder\InstallSettings.xml"
}

function Import-SettingsFile
{
    $settingsFile=InstallSettingFileName

    if (-not (test-path $settingsFile))
    {
        try
        {
            write-host -ForegroundColor White "Creating $settingsFile"
            $global:XMLSettingDoc = New-Object XML
            $global:XMLSettingDoc.InnerXml = "<$XMLNode_Root />"
            Save-SettingFile
        }
        catch
        {
            Write-Divider
            Write-Host -ForegroundColor Red "Problem creating configuration settings file '$settingsFile'"
            Write-Host -ForegroundColor Red $_.FullyQualifiedErrorId
            exit
        }
    }
    else
    {
        try
        {
            $global:XMLSettingDoc = [xml](Get-Content $settingsFile)
        }
        catch
        {
            Write-Divider
            Write-Host -ForegroundColor Red "Problem loading settings file '$settingsFile'"
            Write-Host -ForegroundColor Red $_.FullyQualifiedErrorId
            exit
        }
    }
}

function Save-SettingFile
{
    $fileName=InstallSettingFileName
    $XMLSettingDoc.save($fileName)

    #Load-SettingsFile($DeployPAMFolder)
}

function Get-Parameter
{
    PARAM (
    	[Parameter(Mandatory=$true)] [string]$paramName,
    	[Parameter(Mandatory=$true)] [string]$description,
    	[Parameter(Mandatory=$false)] [boolean]$MandatoryInput=$true,
    	[Parameter(Mandatory=$false)] [string]$default = ""
    )
    Import-SettingsFile

    $root=$XMLSettingDoc.SelectSingleNode("$XMLNode_Root")
    $paramsNode=$root.selectSingleNode($XMLNode_Parameters)
    if ($paramsNode -eq $null)
    {
        $paramsElement=$XMLSettingDoc.CreateElement($XMLNode_Parameters)
        $paramsNode=$root.AppendChild($paramsElement)
        Save-SettingFile
    }

    $paramNode=$paramsNode.SelectSingleNode($paramName)

    if ($xmlCreateCycle)
    {
        if ($paramNode -eq $null)
        {
           $paramsElement=$XMLSettingDoc.CreateElement($paramName)
           $paramsElement.InnerText=""
           $paramNode=$paramsNode.AppendChild($paramsElement)
           Save-SettingFile        
        }
    }
    else
    {
        $currentValue=$paramNode.InnerText
        if ($currentValue -ne "" -and $currentValue -ne $null)
        {
            return $currentValue
        }
        else
        {
            $title="Load value for $paramName"
            $paramValue=read-data -text $description -title $title -default $default -MandatoryInput $MandatoryInput


            $paramNode=$paramsNode.SelectSingleNode($paramName)
            if ($paramNode -eq $null)
            {
               $paramsElement=$XMLSettingDoc.CreateElement($paramName)
               $paramsElement.InnerText=$paramValue
               $paramNode=$paramsNode.AppendChild($paramsElement)
               Save-SettingFile
            }
            else
            {
                if ($paramValue -ne $currentValue)
                {
                   $paramNode.InnerText=$paramValue
                   Save-SettingFile
                }
            }
        }
    }

    return $paramValue

}

function Get-AccountsNode
{
    Import-SettingsFile

    $root=$XMLSettingDoc.SelectSingleNode("$XMLNode_Root")
    $accountsNode=$root.SelectSingleNode($XMLNode_Accounts)
    if ($accountsNode -eq $null)
    {
        write-host -ForegroundColor White "Creating $XMLNode_Accounts node"
        $AccountsElement = $XMLSettingDoc.CreateElement($XMLNode_Accounts)
        $accountsNode = $root.AppendChild($AccountsElement)
        Save-SettingFile
    }
    return $accountsNode

}

function Get-AccountNode($accountName)
{
    $accountsNode = Get-AccountsNode
    $AccountNode=$accountsNode.SelectSingleNode($accountName)

    if ($AccountNode -eq $null)
    {
        write-host -ForegroundColor White "Creating $accountName node" 
        $AccountElement = $XMLSettingDoc.CreateElement($accountName)
        $AccountNode = $accountsNode.AppendChild($AccountElement)

        $AccountNameElement = $XMLSettingDoc.CreateElement($XMLNode_AccountName)
        $AccountNameElement.InnerText=""
        $AccountNode.AppendChild($AccountNameElement)

        $AccountPasswordlement = $XMLSettingDoc.CreateElement($XMLNode_AccountPassword)
        $AccountPasswordlement.InnerText=""
        $AccountNode.AppendChild($AccountPasswordlement)
        Save-SettingFile
    }

    Return $AccountNode
}

function Get-AccountNode($accountName, $CreateIfMissing)
{
    $accountNode=Get-AccountNode -accountName $accountName -CreateIfMissing $CreateIfMissing

    if (-not $xmlCreateCycle)
    {
        $RealAccountName=$accountNode.SelectSingleNode($XMLNode_AccountName).InnerText
        $description=$accountNode.selectSingleNode($XMLNode_AccountDescription).InnerText

        if (($description -eq "") -or ($description -eq $null))
        {
            $description="Enter the $accountName account name"
        }

        if ($CreateIfMissing)
        {
            $description = "$description - this account will be created"
        }
        else
        {
            $description = "$description - this account must exist"
        }

        if ($RealAccountName -eq $null -or $RealAccountName -eq "")
        {
            $keepTrying = $true
            $title="Account name test"
            $default=$accountName
            while ($keepTrying)
            {
                $RealAccountName=readHostWithDefault -msg $description -default $default -title $title -MandatoryInput $true
                $message = [string]::Format("Retrieved RealAccountName {0}", $RealAccountName)
                Write-LogsAndDisplay -EventID 11051 -ErrorLevel Information -message $message
                $accountExists = Confirm-ADaccountExists -samAccountName $RealAccountName
                $message = [string]::Format("Confirm-ADaccountExists for account {0} returned {1}", $RealAccountName, $accountExists)
                Write-LogsAndDisplay -EventID 11051 -ErrorLevel Information -message $message
                $keepTrying = $false

                if (-not $CreateIfMissing)
                {
                    #Account should already exist
                    $keepTrying = -not $accountExists

                    $title="Cound not find account"

                    $default=$RealAccountName
                }
            }

            $keepAskingForPassword=$true
            while ($keepAskingForPassword)
            {
               $SecurePassword = Read-Host "Enter Password for $RealAccountName" -asSecureString  
               $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword)
               $plainTextPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR)

               if ($accountExists)
               {
                    #Check that the password is correct

                    $credsOK = confirm-ADcredentials -samAccountName $RealAccountName -PlainTextPassword $plainTextPassword
                    $message = [string]::Format("Confirm-ADcredentials returned {0} when checking password for {1}", $credsOK, $RealAccountName)
                    Write-LogsAndDisplay -EventID 11051 -ErrorLevel Information -message $message
                    $keepAskingForPassword = -not $credsOK
               }
               else
               {
                    #Create the account
                    New-ADAccount -Password "$plainTextPassword" -AccountName $RealAccountName -InternalAccountName $accountName

                    $keepAskingForPassword=$false
               }
           }
           $accountNode.SelectSingleNode($XMLNode_AccountName).InnerText = $RealAccountName
           $enc=Convert-SecurePasswordToBytes($SecurePassword)

           $accountNode.SelectSingleNode($XMLNode_AccountPassword).InnerText = $enc
           Save-SettingFile

        }

    }
    return $accountNode
}

function Save-Password
{
    PARAM (
	    [Parameter(Mandatory=$TRUE)] [string]$samAccountName,
	    [Parameter(Mandatory=$TRUE)] [SecureString]$SecurePassword
    )

    $accountNode=Get-AccountNode  -accountName $samAccountName -CreateIfMissing $True
    $message = [string]::Format("Retrieved `$accountName")
    $enc=Convert-SecurePasswordToBytes($SecurePassword)

    $accountNode.SelectSingleNode($XMLNode_AccountPassword).InnerText = $enc
    Save-SettingFile

}

function Get-Password
{
        PARAM (
        [Parameter(Mandatory=$true)] [string]$accountName,
        [Parameter(Mandatory=$false)] [boolean]$createIfMissing = $true
    )

    $accountNode=Get-AccountNode -accountName $accountName -CreateIfMissing $createIfMissing
   
    $encPassword=$accountNode.SelectSingleNode($XMLNode_AccountPassword).InnerText
    $password=Convert-PasswordBytesToPlaintext($encPassword)

    $credsOK=$false
    while (-not $credsOK)
    {
        if ($password -ne $null)
        {
            $credsOK = confirm-ADcredentials -samAccountName $accountName -PlainTextPassword $password
        }
        else
        {
            $message = [string]::Format("Credentials were incorrect and `$password is NULL for user {0}", $accountName)
            Write-LogsAndDisplay -EventID 11051 -ErrorLevel Error -message $message
        }
        if (-not $credsOK)
        {
           $password = Read-Host "Please provide the password for $Global:AccountDescription" -AsSecureString
           $message = [string]::Format("The password for {0} is {1} characters.", $Global:AccountDescription, $password.Length)
           Write-LogsAndDisplay -EventID 11051 -ErrorLevel Error -message $message

           $message = [string]::Format("Storing password for {0}.", $accountName)
           Write-LogsAndDisplay -EventID 11051 -ErrorLevel Error -message $message

           Save-Password -samAccountName $accountName -SecurePassword $password

           $encPassword=$accountNode.SelectSingleNode($XMLNode_AccountPassword).InnerText
           $plainTextPassword=Convert-PasswordBytesToPlaintext -encPassword $password

           $credsOK = confirm-ADcredentials -samAccountName $accountName -PlainTextPassword $plainTextPassword
        }
    }

    return $password
}

function Get-AccountName
{
    PARAM (
        [Parameter(Mandatory=$true)] [string]$accountName,
        [Parameter(Mandatory=$false)] [boolean]$createIfMissing = $true
    )


    $accountNode=Get-AccountNode -accountName $accountName -CreateIfMissing $createIfMissing

    return $accountNode.SelectSingleNode($XMLNode_AccountName).InnerText
}

Function Get-AccountDescription
{
  PARAM (
        [Parameter(Mandatory=$true)] [string]$accountName
  )

  $accountNode=Get-AccountNode -accountName $accountName -CreateIfMissing $createIfMissing
  $name = $accountNode.SelectSingleNode($XMLNode_AccountName).InnerText
  $desc = $accountNode.SelectSingleNode($XMLNode_AccountDescription).InnerText
  if ($desc -eq $null -or $desc -eq"")
  {
    $message = [string]::Format("The `$XMLNode_AccountDescription.InnerText value for accountName {0} is either NULL or EMPTY", $accountName)
    Write-LogsAndDisplay -EventID 11052 -ErrorLevel Warning -message $message
    return "Account name $name"
  }
  else
  {
    $message = [string]::Format("The `$XMLNode_AccountDescription.InnerText value for accountName {0} is '{1}'", $accountName, $desc)
    Write-LogsAndDisplay -EventID 11052 -ErrorLevel Information -message $message
    return "$desc ($name)"
  }

}

Function Get-ADuserObject
{
    PARAM (
	    [Parameter(Mandatory=$true)] [string]$samAccountName 
    )

    $tryAgain=$true
    $triedEnough=$false
    while ($tryAgain)
    {
        try
        {
            $User = Get-ADUser -Filter {sAMaccountName -eq $samAccountName}
            $tryAgain=$false
        }
        catch
        {
            if ($_.Exception -contains 'Cannot find an object')
            {
                $user=$null
                $tryAgain=$false
            }
            else
            {
                if (-not $triedEnough)
                {
                    #During a re-boot, some services as not ready - give it some time
                    $retvalue=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information " - Error: $_.Exception"
                    $retvalue=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information "Waiting for services to start"
                    Start-Sleep -Seconds 5
                    $triedEnough = $True
                }
                else
                {
                    return "OK we tried"
                }
            } #if
        } #try
    } #while

    return $user
}

Function Confirm-ADaccountExists
{
    PARAM (
	    [Parameter(Mandatory=$true)] [string]$samAccountName 
    )


    $retvalue=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message  "Check AD account $samAccountName exists"

    $User = Get-ADuserObject -sAMAccountName $samAccountName

    if ($user -ne $null)
    {
        $message = [string]::Format("Account {0} exists", $samAccountName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message  $message
        Return $True
    }
    else
    {
        $message = [string]::Format("Account {0} does not exist!!!", $samAccountName)
        $retvalue=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message $message
        Return $False
    }
}

Function Confirm-ADcredentials
{
    PARAM (
	    [Parameter(Mandatory=$true)] [string]$samAccountName,
	    [Parameter(Mandatory=$true)] [string]$PlainTextPassword
    )

    if ($BeingRebooted)
    {
        $message = [string]::Format("Skipping AD account check for {0} during re-boot", $samAccountName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message  $message
        return $True
    }
    $message = [string]::Format("Check AD account {0} password is valid", $samAccountName)
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message $message

    $accountExists = Confirm-ADaccountExists -samAccountName $samAccountName
    if ($accountExists)
    {
        $domain = new-object directoryservices.directoryentry("",$samAccountName,$PlainTextPassword)

        ###$PlainTextPassword='PA$$w0rd'

        ###$domain = new-object directoryservices.directoryentry("",$samAccountName,$PlainTextPassword)
        $credsValid = $domain.psbase.name -ne $null
        if ($credsValid)
        {
            $message = [string]::Format("Account credentials valid for {0}", $samAccountName)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -Message $message
        }
        else
        {
            $message = [string]::Format("Account credentials NOT VALID for {0}", $samAccountName)
            $retvalue=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -Message  "Account credentials invalid!!!"
        }

        return $credsValid

    }
    else
    {
        return $false
    }
}

Function New-ADAccount($Password, $AccountName, $InternalAccountName)
{
    $SecurePassword = ConvertTo-SecureString "$password" –asplaintext –force
    $User = Get-ADuserObject -sAMAccountName $AccountName
    If ($User  -eq $Null) 
    {
        $message = [string]::Format("Account {0} does not exist in AD. Attempting to create...", $AccountName)
        Write-LogsAndDisplay -EventID 11009 -ErrorLevel Information -message $message
    
        $Description="Service account for MIM/PAM "
        New-ADUser –SamAccountName $accountName –name $accountName -surname $accountName -GivenName $AccountName -DisplayName $AccountName -Description $Description -AccountPassword $SecurePassword
    }
    Else 
    {
        write-host -ForegroundColor white "User $AccountName already exists in AD"
    }

    if ($internalAccountName -ne $AccountName_Admin)
    {
        Set-ADAccountPassword –identity $accountName –NewPassword $SecurePassword
        Set-ADUser –identity $accountName –Enabled 1 –PasswordNeverExpires 1
    }

    if ($AccountName -eq $accountName_MIMService -or $accountName -eq $accountName_SharePoint)
    {
        write-host -ForegroundColor white "Adding $AccountName to Domain Admins"
    
        Add-ADGroupMember "Domain Admins" -Members $accountName -ErrorAction SilentlyContinue
    }
}

function Import-Credentials
{
    PARAM (
        [Parameter(Mandatory=$true)] [boolean]$SharePointInstall 
    )
    $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_Admin
    $Global:AdminAccount=get-AccountName -accountName $AccountName_Admin -createIfMissing $false
    $Global:AdminPassword = get-Password -accountName $AccountName_Admin  -createIfMissing $false

    $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_SharePoint
    $Global:SharePointAccountName=get-AccountName  -accountName $AccountName_SharePoint
    $Global:SharePointAccount_Password=get-Password  -accountName $AccountName_SharePoint

    $Global:PAM_REST_API_APPPOOL_ACCOUNT_NAME=$SharePointAccountName
    $Global:PAM_REST_API_APPPOOL_ACCOUNT_PASSWORD=$SharePointAccount_Password

    $Global:SQLSERVER_SERVER=Get-Parameter -paramName $XMLNode_SQL_Server -description "SQL server name"

    $Global:PortalURL=Get-Parameter -paramName $XMLNode_PortalURL -description "Portal URL" -default "http://$ServerName.$Domain.local"

    if (-not $SharePointInstall)
    {
        #Find/Create the other attributes required for MIM-PAM

        $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_MIMMonitor
        $Global:PAM_MONITORING_SERVICE_ACCOUNT_NAME=get-AccountName  -accountName $AccountName_MIMMonitor
        $Global:PAM_MONITORING_SERVICE_ACCOUNT_PASSWORD = get-Password  -accountName $AccountName_MIMMonitor

        $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_MIMComponent
        $Global:PAM_COMPONENT_SERVICE_ACCOUNT_NAME=get-AccountName  -accountName $AccountName_MIMComponent
        $Global:PAM_COMPONENT_SERVICE_ACCOUNT_PASSWORD=get-Password  -accountName $AccountName_MIMComponent

        $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_MIMService
        $Global:PAM_SERVICE_ACCOUNT_NAME=get-AccountName  -accountName $AccountName_MIMService
        $Global:PAM_SERVICE_ACCOUNT_PASSWORD=get-Password  -accountName $AccountName_MIMService

        $Global:AccountDescription =Get-AccountDescription -accountName $AccountName_SyncMAAccount
        $Global:SYNC_ADMIN_ACCOUNT_NAME=get-AccountName  -accountName $AccountName_SyncMAAccount
        $Global:SYNC_ADMIN_ACCOUNT_PASSWORD=get-Password  -accountName $AccountName_SyncMAAccount

        $Global:MAIL_SERVER=Get-Parameter -paramName $XMLNode_Corp_Server -description "Corporate server name"
        $Global:LoggingLevel=Get-Parameter -paramName $XMLNode_LoggingLevel -description "Logging level 1:Information, 2:Warnings, 3:Errors" -default "3"
    }

}

function Initialize-CommonMethods
{
    PARAM(
        [Parameter(Mandatory=$true)] [string] $DeployPAMFolder,
        [Parameter(Mandatory=$true)] [boolean] $SharePointInstall,
        [Parameter(Mandatory=$false)] [boolean] $CreateIfMissing = $true
    )

    #Install-WindowsFeature RSAT-AD-Powershell
    $FeatureADPowershell = Get-WindowsFeature RSAT-AD-PowerShell

    if ($FeatureADPowershell.installstate -eq 'installed') {

        Write-Host "RSAT-AD-Powershell install state is Installed: "$FeatureADPowershell.installstate

    } else {

        Write-Host "RSAT-AD-Powershell install state is not Installed. Installing "$FeatureADPowershell.installstate
        Install-WindowsFeature RSAT-AD-Powershell

    }

	#Load ActiveDirectory if not present
    $ModuleAD = Get-Module ActiveDirectory

    if ($ModuleAD.name -eq "ActiveDirectory") {

        write-host "ActiveDirectory is Installed"

    } else {

        write-host "ActiveDirectory is not Installed, installing"
        Import-Module ActiveDirectory

    }
    

    #Set this to $true to create an empty XML settings file
    $xmlCreateCycle=$false

    #Load VS for use with message box
    [System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic') | Out-Null

    $global:DeployPAMFolder=$DeployPAMFolder


    $Global:BeingRebooted = IsRestartRequired

 #Settings file XML node names
    $XMLNode_Root='Configuration'
    $XMLNode_Accounts='Accounts'
    $XMLNode_Account='Account'
    $XMLNode_AccountName='AccountName'
    $XMLNode_AccountDescription='Description'
    $XMLNode_AccountPassword='AccountPassword'
    $XMLNode_Parameters='Parameters'
    $XMLNode_SQL_Server="sqlServer"
    $XMLNode_Corp_Server="corpServer"
    $XMLNode_PortalURL="portalURL"
    $XMLNode_LoggingLevel="loggingLevel"

 #Base account names and tags in the settings XML file
    $AccountName_Admin="Administrator"
    $AccountName_MIMmonitor="MIMmonitor"
    $AccountName_MIMcomponent= "MIMcomponent"
    $AccountName_MIMSync= "MIMSync"
    $AccountName_MIMService= "MIMService"
    $AccountName_SharePoint="SharePoint"
    $AccountName_SqlServer="SqlServer"
    $AccountName_BackupAdmin= "BackupAdmin"
    $AccountName_SyncMAAccount="MIMma"

#Logging data
# TODO - from MPL - This section doesn't belong in CommonFunctions
##################################################################
    $Global:EventLogName = 'Application'
    if ($SharePointInstall)
    {
        $Global:EventLogSource = 'MIM SharePoint install'
    }
    else
    {
        $Global:EventLogSource = 'MIM-PAM install'
    }
##################################################################

    $Global:logFilePath = "$DeployPAMFolder\logs\"
    $global:LogFileName = Get-LogFileName -filename $EventLogSource -path "$logFilePath"

    $res = Register-EventLogSource -eventLogName $EventLogName -eventLogSource $EventLogSource

    if (-not (Test-Path -path "$logFilePath"))
    {
        try
        {
            New-Item -Path "$logFilePath" -Type directory | out-null
        }
        catch
        {
            $reply=Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -Message  "Error creating log folder $logFilePath"
        }
    }


    $Global:Domain=$env:USERDOMAIN
    $Global:DomainAccount="$Domain\$AdminAccount"
    $Global:DomainAccountSecondaryOwnerAlias=$DomainAccount
    $Global:SharePointAccount="$Domain\$SharePointAccountName"
    $Global:ServerName=$env:COMPUTERNAME

    Import-Credentials -SharePointInstall $SharePointInstall

}

function readHostWithDefault
{
    PARAM (
        [Parameter(Mandatory=$true)] [string]$msg,
        [Parameter(Mandatory=$true)] [string]$title,
        [Parameter(Mandatory=$true)] [string]$default,
        [Parameter(Mandatory=$true)] [boolean]$MandatoryInput

    )
        $answer=read-data -text $msg -title $title -default $default -MandatoryInput $MandatoryInput
        return $answer
}


Export-ModuleMember -Function ClearRestartRequired
Export-ModuleMember -Function Confirm-ADaccountExists
Export-ModuleMember -Function Confirm-ADcredentials
Export-ModuleMember -Function Confirm-LocalSession
Export-ModuleMember -Function Convert-PasswordBytesToPlaintext
Export-ModuleMember -Function Convert-SecurePasswordToBytes
Export-ModuleMember -Function Get-AccountDescription
Export-ModuleMember -Function Get-AccountName
Export-ModuleMember -Function Get-AccountNode
Export-ModuleMember -Function Get-AccountsNode
Export-ModuleMember -Function Get-ADuserObject
Export-ModuleMember -Function Get-LogFileName
Export-ModuleMember -Function Get-NetBIOSName
Export-ModuleMember -Function Get-AccountNode
Export-ModuleMember -Function Get-Parameter
Export-ModuleMember -Function Get-Password
Export-ModuleMember -Function Initialize-ActiveDirectoryModule
Export-ModuleMember -Function Initialize-CommonMethods
Export-ModuleMember -Function InstallSettingFileName
Export-ModuleMember -Function IsRestartRequired
Export-ModuleMember -Function Import-Credentials
Export-ModuleMember -Function Import-SettingsFile
Export-ModuleMember -Function New-ADAccount
Export-ModuleMember -Function OurRegistryNode
Export-ModuleMember -Function Read-Data
Export-ModuleMember -Function readHostWithDefault 
Export-ModuleMember -Function Read-HostWithDefault
Export-ModuleMember -Function Register-EventLogSource
Export-ModuleMember -Function Save-SettingFile
Export-ModuleMember -Function Set-RestartRequired
Export-ModuleMember -Function Show-Progress
Export-ModuleMember -Function Save-Password
Export-ModuleMember -Function Test-AdminAccount
Export-ModuleMember -Function Test-DomainJoin
Export-ModuleMember -Function Test-IsInDomain
Export-ModuleMember -Function Test-IsStaticIP
Export-ModuleMember -Function Test-IsVersion2012R2orHigher
Export-ModuleMember -Function Test-IsWindowsServer
Export-ModuleMember -Function Test-PasswordComplexity
Export-ModuleMember -Function Test-ServerOSVersion
Export-ModuleMember -Function Write-Divider
Export-ModuleMember -Function Write-LogFile
Export-ModuleMember -Function Write-LogsAndDisplay
