#Name: Initialize-ADDSForest.ps1 (renamed from New-ADDSforest.ps1 on 22-July-2015)
#Version: V02.08.00 - lower number used for internal release, next higher for MS release, top digit for major version change
#Release date/time: 2015.09.17
#Description:
#TLG ref: Step 2 - Prepare PRIV domain controller

<#
.SYNOPSIS 

    Promotes the local server to become the first domain controller in a new Active Directory forest.	The local administrator account will become a domain administrator in the new forest.

.DESCRIPTION

    This script promotes the local system to become the first domain controller in a new Active Directory forest.  Necessary services like DNS will be installed as well.  The local Adminsitrator account on the server will become a member of Domain Admins in the new forest.
    

.PARAMETER DomainName

		Fully-qualified DNS domain name. This parameter is required.

.PARAMETER DatabasePath

        Fully-qualified path to local folder where Directory database files (NTDS) are to be stored.  Defaults to $env:SystemRoot\NTDS if omitted.

.PARAMETER SysvolPath

        Fully-qualified path to local folder where Sysvol files are to be stored.  Defaults to $env:SystemRoot\Sysvol if omitted.

.PARAMETER LogPath

        Fully-qualified path to local folder where Log files are to be stored.  Defaults to $env:SystemRoot\Logs if omitted.

.PARAMETER DomainMode

        The functional level for the new domain.  This defaults to Windows Server 2012R2 mode if omitted.

.PARAMETER ForestMode

        The functional level for the new forest.  This defaults to Windows Server 2012R2 mode if omitted.

.EXAMPLE

    New-ADDSForest.ps1 -DomainName priv.local

    Promotes the current machine to be the first domain controller in a new Active Directory forest called "priv.local"

.EXAMPLE

    New-ADDSForest.ps1 -DomainName priv.local -DatabasePath E:\NTDS -SysvolPath E:\SYSVOL -LogPath F:\LOGS

    Promotes the current machine to be the first domain controller in a new Active Directory forest called "priv.local".  The specified paths will be used for the Database, SYSVOL and Logs.

#>

#
# Revision history
# 2.6.01 -15-Sep-2015 - Fixed handling of passwords that fail Test-PasswordComplexity
# (ocg\mike)          - NEW FEATURE: Creates local users by caling Create-MIMPAMLocalUsers.ps1
# 2.0.7 - 02-Sep-2015 - Added additional logging to aid in post-mortem analysis
# (ocg\mike)          - Fixed bad NetBIOS parameter name in call to Test-ForestCreation function
#                     - Now using Get-NetBIOS function in CommonFunctions.psm1
#                     - Deleted old version of Test-IPAddress function
#                     - Changed method of getting Restore Mode Password from Get-Credentials to 
#                     -    Read-Host to prevent user from dismissing Get-Credentials box
#                     - Added check for zero-length Directory Services Restore Mode password
# 2.0.6 - 19-Aug-2015 - Changed Logging from Logging.psm1 to CommonFunctions.ps1
# (ocg\mike)          - Added password complexity test for Restore Mode Password
#                     -   At least 15 characters, one UPPER, one lower, one non-alpha.
# 2.0.5 - 14-Aug-2015 - Making displayed colors consistent.
# (ocg\mike)          - Removing commented code
# 2.0.4 - 11-Aug-2015 - Corrected error in restart logic
# (ocg\mike)
# 2.0.3 - 10-Aug-2015 - Assigned $tfc status to $message variable. Previously assigned value was incorrectly displayed. (Line 970)
# (ocg\mike)          - Chaged $tfc status report to Information instead of Error (line 972)
#                     - removed reference to unassinged variable $eventLogName in Clean Up After Using section
#  
# 2.0.2 - 05-Aug-2015 - Corrected return values for Test-IsADDomainServicesFeatureAvailable and Test-IsDNSFeatureAvailable
# (ocg\mike)
# 2.0.1 - 31-Jul-2015 - Changed name ocgErrorLevel enumeration to tdErrorLevel. That was the last reference to OCG.The td is for TypeDefinition. (Arik feedback #1)
# (ocg\mike)          - Fixed desscriptions of NTDS, Sysvol and Logs to match correct default path $env:SystemRoot\ (Arik feedback #2)
#                     - Fixed error message by usin[string]::format("{0} exists", $DatabasePath) (Arik feedback #3)
#                     - Changed "C:\" to $env:SystemDrive\ (Arik feedback #4)
#                     - Changed Safe Mode Administrator to Directory Service Restore Mode in numerous locations. Only reference to SafeModeAdministrator is in
#                           required parameter names to Microsoft's PowerShell install-ADDSForest and Test-ADDSForestInstallation (Arik feedback #7, 2nd bullet)
#                     - Did not correct for case where blank Directory Services Restore Mode is accepted because subsequent test aborts install if blank (Arik feedback #7, first bullet)
#                     - Corrected Logging module errors (came about because correct version of module was not submitted with script) (Arik feedback #8)
#                     - Revised function Test-IsStaticIPaddress to correctly identify IP-enabled adapters and ignore loopback adapter (Arik feedback #9)
#                            Old function renamed to Test-IsStaticIPAddress_Old for reference. Can be deleted after revised function is tested thoroughly.
#                     - Corrected operator in Get-SafeModeAdministratorPassword (function now named Get-DirectoryServiceRestoreModePassword) to use -eq (Arik feedback #10)
#                     - Corrected inconsistent message colors (Arik feedback #12)
#                     - Corrected multiple answers in final report (Arik feedback #13)
#                     - Removed stray False return value (Arik feedback #14)
#                     - Added NetBIOSName as parameter to Install-ADDSForest (Arik feedback #15)
#                     - Added DNS validation for correct characters and form in new Test-DNSDomainName function (Arik feedback $16, first item)
#                     - Added code to handle case of $tfc.Status -ne "Success" (Arik feedback 16, 2nd item)
#
#                     STILL TO COMPLETE: Password complexity validation (need to directly handle secure strings without persisting decryted password as variable) (Arik feedback #7, first bullet)
#                                        DNS Delegation - it adds some new conditions that failed initial testing. (Arik feedback #11)
#                                        
#
# 2.0.0 - 23-Jul-2015 - Fixed module connection issues due to rename process.
# (ocgj\burtt)        - burtt & kenh ran successfully on different VMs.
#
# 1.3.3 - 22-JUL-2015  Changed name of script to "New-ADDSForest.ps1" to align with verb-noun naming best-practice
# (OCG\kenh)           Added comment-based help block 
#                      Changed hard-coded file path for OCGLogging.psm1 module to assume it's in the local directory

# 1.3.2 - 20150720 - Changed Install paths for Database, Sysvol and Logs to use "$env:SystemRoot" instead of hard-coded c:\
#                    Changed location of OCGLogging.psm1 to C:\Project folder
#                    Replaced LogAndDisplay with Write-LogsAndDisplay to be compatable with approved PowerShell verbs
#                    Replaced IsDomainController with Test-IsDomainController to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsVersion2012R2orHigher to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsWindowsServer to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsVersion2012R2orHigher to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsInDomain to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsStaticIP to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsDatabasePathValid to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsSysvolPathValid to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsLogPathValid to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsDomainSingleRoot to be compatable with approved PowerShell verbs
#                    Replaced with Get-NetBIOSName to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsADDomainServicesFeatureAvailable to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsADDomainServicesFeatureInstallationRequired to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsDNSFeatureAvailable to be compatable with approved PowerShell verbs
#                    Replaced with Test-IsDNSFeatureInstallationRequired to be compatable with approved PowerShell verbs
#                    Replace with Install-ADDomainServices  to be compatable with approved PowerShell verbs
# Install-DNS
# 1.3.2 - 20150720 - Changed Install paths for Database, Sysvol and Logs to use "$env:SystemRoot" instead of hard-coded c:\
#                    Changed location of OCGLogging.psm1 to C:\Project folder
# 1.3.1 - 20150719 - Removed filter by name when checking for IP Address in IsStaticIP function
#                    | where {($_.InterfaceAlias -like 'Ethernet*') -and ($_.InterfaceAlias -notlike 'Loopback*')}
# 1.3.0 - 20150702 - added header and revised some of the messagesso they are more accurate/
#                    revised parameter statements by defining types and mandatory settings
#                    added block for PowerShell help syntax
############################
# Create Active Directory Forest
#
# Creates a new Active Directory by making a computer running Windows Server 2012 R2 into the first Domain Controller in a new forest.
############################
############################
# SCRIPT PARAMETERS
############################
PARAM(
	[Parameter(Mandatory=$true)] [string]$DomainName,
#	[Parameter(Mandatory=$false)] [string]$DomainControllerName,           # Not used elsewhere in the script - ocg\kenh
	[Parameter(Mandatory=$false)] [string]$DatabasePath = "$env:SystemRoot\NTDS",
	[Parameter(Mandatory=$false)] [string]$SysvolPath = "$env:SystemRoot\SYSVOL",
	[Parameter(Mandatory=$false)] [string]$LogPath = "$env:SystemRoot\Logs",
	[Parameter(Mandatory=$false)] [string]$DomainMode = "Win2012R2",
	[Parameter(Mandatory=$false)] [string]$ForestMode = "Win2012R2",
#	[Parameter(Mandatory=$false)] [boolean]$VerboseLogging = $true         # Not used elsewhere in the script - ocg\kenh
#BinilK - Added these for DNS delegation
	[Parameter(Mandatory=$false)] [boolean]$CreateDNSDelegation = $false,
	[Parameter(Mandatory=$false)] [PSCredential]$DNSDelegationCredential = $null
)

################################
# SCRIPT ENUMERATIONS
################################
Add-Type -TypeDefinition @"
    public enum tdErrorLevel
    {
        informational,
        warning,
        error
    }
"@

################################
# SCRIPT FUNCTIONS
################################
Function Import-CommonFunctions
{
    Param(
        [string]$deployFolder,
        [string]$moduleName
        )
        $bReturn = $false
        $error.Clear()
        $modulePath = $deployFolder + '\' + $moduleName
        Write-Host -ForegroundColor Green Attempting to load module $modulePath
        if (Test-Path $modulePath)
        {
            Try
            {
                import-module $modulePath -ErrorAction SilentlyContinue
                Write-Host -ForegroundColor Green Successfully loaded module $modulePath
                $bReturn = $true
            }
            Catch
            {
                if ($error.Count -gt 0)
                {
                Write-Host -ForegroundColor Red $error[0].Exception.Message
                # TODO: Should we exit or continue?
                exit
                }
            }
        }
        else
        {
            $message = [string]::Format("Did not find {0} in {1}", $moduleName, $deployFolder)
            Write-host -ForegroundColor Red $message
            # TODO: Should we exit or continue?
            exit
        }
        return $bReturn
}

Function Test-IsWindowsServer
{
	$retval = $false
	# check operating system is Windows 2012 R2
	# Get-WmiObject has been deprecated. We should use Get-CimInstance instead.

	$cim = Get-CimInstance Win32_OperatingSystem
	# make sure running on server and not workstation (version numbers are same)
	If ($cim.Caption.IndexOf("Server") eq 0)
	{
        $message =  [string]::Format("Windows Server 2012 R2 operating system is required. Installed operating system is {0}", $cim.Caption)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
    	$retval = $false;
	}
	else
	{
        $message = [string]::Format("Installed operating system is {0}", $cim.caption)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsVersion2012R2orHigher
{
    $cim = Get-CimInstance Win32_OperatingSystem
	$retval = $false
	$arrVer = $cim.version.Split(".")
	# we should have an array of at least three elements
    # the [0] element contains the "major" version number
	# the [1] element contains the "minor" version number
	# the [2] element contains the build number
	# NB: Server and Workstation cannot be differentiated by this numbering scheme.
	If ($arrVer.Length gt 1)
	{
		If (([Int32]$arrVer[0]) ge 6)
		{
		# we have Windows 2008 or later. This is minimum major version.

			If (([Int32]$arrver[1]) ge 3)
			{
				# we have Windows 2012 R2 or later
				# 0 = Server 2008
				# 1 = Server 2008 R2
				# 2 = Server 2012
				# 3 = Server 2012 R2 -- this is our minimum minor version
                $message = [string]::Format("Installed operating system is {0}.", $cim.Version)
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
				$retval = $true
			}
			Else
			{
                $message = [string]::Format("Privileged domain requires Microsoft Windows 2012 R2 or later. Installed operating system is {0}.", $cim.Caption)
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
				$retval = $false
			}
		}
	}
	return $retval
}

Function Test-IsInDomain
{
    # TODO: Move the error and exit logic out of this function so it is handled by caller
	$retval = $false
	# check to see if computer is joined to a domain
	# TODO: convert this to use CIM namespace
	$wmi = (gwmi win32_computersystem)
	if ($wmi.partofdomain -eq $true)
	{
		$message = 'Machine is part of the existing domain. It cannot become the first domain controller is a new forest.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		exit
	}
	else
	{
		$message = [string]::format("This computer, {0}, is not part of a domain. It is ready to become the first domain controller in a new forest.", $env:COMPUTERNAME)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		return $true
	}
	return $retval
}

Function Test-IsDatabasePathValid
{
	$retval = $false
	$res = Test-Path $DatabasePath
	if ($res -eq $false)
	{
        $message = [string]::Format("Database path {0} does not exist. Trying to create...", $DatabasePath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    $ni = New-Item -ItemType directory -Path $DatabasePath
	    if ((Test-Path $DatabasePath) -eq $false)
	    {
	        $message = 'Failed to create DatabasePath at ' + $DatabasePath
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
            write-host -fore red $message
            Write-EventLog LogName $eventLogName Source $eventLogSource EntryType Information EventID 11005 -Category 0 Message $message
            Write-LogFile -logFileName $global:logFileName -message $message
	        $retval = $false
	    }
	    else
	    {
            $message = [string]::format("Database path {0} successfully created.", $DatabasePath)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    	$retval = $true
	    }
	}
	else
	{
        $message = [string]::format("{0} exists", $DatabasePath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsSysvolPathValid
{
	$retval = $false
	$res = Test-Path $SysvolPath
	if ($res -eq $false)
	{
        $message = [string]::Format("Sysvol path {0} does not exist. Trying to create...", $SysvolPath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    $ni = New-Item -ItemType directory -Path $SysvolPath
	    if ((Test-Path $SysvolPath) -eq $false)
	    {
            $message = [string]::format("Failed to create SysvolPath at {0}", $SysvolPath)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
	        $retval = $false
	    }
	    else
	    {
            $message = [string]::Format("Sysvol path {0} successfully created", $SysvolPath)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    	$retval = $true
	    }
	}
	else
	{
        $message = [string]::Format("{0} exists.", $SysvolPath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsLogPathValid
{
	$retval = $false
	$res = Test-Path $LogPath
	if ($res -eq $false)
	{
        $message = [string]::Format("Log path {0} does not exist. Trying to create...", $LogPath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    $ni = New-Item -ItemType directory -Path $LogPath
	    if ((Test-Path $LogPath) -eq $false)
	    {
            $message = [string]::Format("Failed to create LogPath at {0}", $LogPath)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
	        $retval = $false
	    }
	    else
	    {
            $message = [string]::Format("Log path {0} successfully created", $LogPath)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	    	$retval = $true
	    }
	}
	else
	{
        $message = [string]::Format("{0} exists", $LogPath)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsDomainSingleRoot
{
	$retval = $false
	if ($DomainName.IndexOf(".") -lt 0)
	{
        $message = 'Single root domain is not supported.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		$retval = $false
	}
	else
	{
        $message = [string]::Format("Domain is multipart: {0}.", $DomainName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

#Function Get-NetBIOSName-OLD
#{
#	$NetBIOSName = $DomainName.Substring(0, $DomainName.IndexOf(".")).ToUpper()
#	if ($NetBIOSName.length -gt 15)
#	{
#		$NetBIOSName = $NetBIOSName.Substring(0, 15)
#        $message = [string]::Format("Left-most segment of {0} is longer than 15 characters. NetBIOSName cannot be longer than 15 characters. Trimming NetBIOSName to 15 characters: {1}.", $DomainName, $NetBIOSName)
#        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
#	}
#	elseif ($NetBIOSName.length -eq 0)
#    {
#        $message = [string]::Format("Did not successfully parse a NetBIOS name from {0}.", $DomainName)
#        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
#        Exit
#    }
#    else
#	{
#        $message = [string]::Format("NetBIOSName is {0}", $NetBIOSName)
#        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
#	}
#	return $NetBIOSName
#}

Function Test-IsADDomainServicesFeatureAvailable
{
	$retval = $false
	# ensure AD-Domain-Services is available
	$ADDSAvailable = $false
	$wf = Get-WindowsFeature AD-Domain-Services
	if ($wf -eq $null)
	{
        $message = 'AD-Domain-Services feature is not available'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		$retval = $false
	}
	if ($wf.Installed -eq $false)
	{
		if ($wf.InstallState -eq 'Available')
		{
            $message = 'AD-Domain-Services feature is available'
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
			$retval = $true
		}
		else
		{

            $message = [string]::Format("AD-Domain-Services InstallState is {0}", $wf.InstallState)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
			# In truth, not sure how to react if we get here
		}
	}
    return $retval
}

Function Test-IsADDomainServicesFeatureInstallationRequired
{
	$retval = $true
	# this is an expensive call and it is redundant
	$wf = Get-WindowsFeature AD-Domain-Services
	if ($wf.Installed -eq $true)
	{
        $message = [string]::format("AD-Domain-Services feature is already installed. AD-Domain-Services InstallState is {0}", $wf.InstallState)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $false
	}
	else
	{
        $message = 'AD-Domain-Services installation is required'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Test-IsDNSFeatureAvailable
{
	$retval = $false
	# ensure DNS is available
	$DNSAvailable = $false
	$wf = Get-WindowsFeature DNS
	if ($wf -eq $null)
	{
        $message = 'DNS feature is not available.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		$retval = $false
	}
	if ($wf.Installed -eq $false)
	{
		if ($wf.InstallState -eq 'Available')
		{
            $message = 'DNS feature is available.' 
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
			$retval = $true
		}
		else
		{
            $message = [string]::Format("DNS InstallState is {0}", $wf.InstallState)
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
			# In truth, not sure how to react if we get here
			if ($wf.InstallState -eq 'UninstallPending')
			{
                $message = 'A REBOOT IS REQUIRED before DNS role can be installed.'
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
				$retval = $false
			}
			else
			{
                $message = [string]::Format("DNS role InstallState is {0}.", $wf.InstallState)
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
				$retval = $false
			}
		}
	}
    return $retval
}

Function Test-IsDNSFeatureInstallationRequired
{
	$retval = $true
	# this is an expensive call and it is redundant
	$wf = Get-WindowsFeature DNS
	if ($wf.Installed -eq $true)
	{
        $message = [string]::Format("DNS feature is already installed. InstallState is {0}.", $wf.InstallState)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $false
	}
	else
	{
        $message = 'DNS installation is required.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Install-ADDomainServices
{
	$retval = $false
    $message = [string]::Format("Calling Install-WindowsFeature AD-Domain-Services -IncludeAllSubFeature IncludeManagementTools")
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	$result = Install-WindowsFeature AD-Domain-Services -IncludeAllSubFeature IncludeManagementTools
	if ($result.Success -ne $true)
	{
        $message = 'Could not install the AD-Domain-Services role.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		# exit
	}
	else
	{
        $message = 'Successfully added AD-Domain-Services role with all sub features and Management Tools'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Install-DNS
{
	$retval = $false
    $message = [string]::Format("Install-WindowsFeature DNS -IncludeAllSubFeature IncludeManagementTools")
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	$result = Install-WindowsFeature DNS -IncludeAllSubFeature IncludeManagementTools
	if ($result.Success -ne $true)
	{
        $message = 'Could not install the DNS role.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
		# exit
	}
	else
	{
        $message = 'Successfully added DNS role with all sub features and Management Tools.'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
		$retval = $true
	}
	return $retval
}

Function Get-DirectoryServiceRestoreModePassword
{
	# Test-ADDSForestInstallation will both prompt for password so gather at beginning of script
	#-------------------

	# write-host Provide a Directory Service Restore Mode Password
	$bFirst = $true
	Do
	{
        $error.Clear()
		if ($bFirst -eq $false)
		{  
			# $cred = Get-Credential -Message 'PASSWORDS DID NOT MATCH. Enter the Directory Service Restore Mode password:' -user RestoreModePwd
            $SecurePassword = Read-Host "PASSWORDS DID NOT MATCH. Enter Directory Service Restore Mode Password: " -asSecureString  
            # write-host Breakpoint
		}
		else
		{
            $SecurePassword = Read-Host "Enter Directory Service Restore Mode Password: " -asSecureString  
            # $cred = Get-Credential -Message 'Enter the Directory Service Restore Mode password:' -user RestoreModePwd
            While ($SecurePassword.Length -eq 0)
            {
                $SecurePassword = Read-Host "A PASSWORD IS REQUIRED. Enter Directory Service Restore Mode Password:  " -asSecureString  
            }
            $message = [string]::Format("Calling Test-PasswordComplexity `$SecurePassword") 
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
            if ($SecurePassword.Length -gt 0)
            {
                Try
                {
                    $res = Test-PasswordComplexity -passwd $SecurePassword
                    $bFirst = $false
                }
                Catch
                {
                    Write-Host -ForegroundColor Red Error calling Test-PasswordComplexity. Exiting....
                    Write-Host -ForegroundColor Red $error[0].CategoryInfo
                    Write-Host -ForegroundColor Red $error[0].Exception.Message
                    Exit
                }
            }
            if ($res -eq $false)
            {
                $message = [string]::Format("Restore Mode Password does not meet complexity requirements")
                #write-host -fore red Restore Mode Password does not meet complexity requirements
                Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
                $bFirst = $true
            }
        }
        if ($bFirst -eq $false)
        {
            $SecurePassword1 = Read-Host "Re-enter Directory Service Restore Mode Password: " -asSecureString  
            # $cred1 = Get-Credential -Message 'Re-enter the Directory Service Restore Mode password to confirm:' -user RestoreModePwd
            if ($SecurePassword1 -ne $null)
            {
                $bFirst = $false
                $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword)
                $BSTR1 = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword1)
            }
        }	    	    
    } Until (($bFirst -eq $false) -AND ([System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR) -eq [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR1)))
	# create a PSCredential object
    $cred = New-Object System.Management.Automation.PSCredential -ArgumentList 'DSRMPassword', $SecurePassword
    return $cred
}

function Test-ForestCreation
{
	PARAM( [PSCredential]$Credential,
    [string]$NetBIOSName )

    $message = [string]::Format("Calling Test-ADDSForestInstallation -DomainName {0} -InstallDNS -DomainMode {1} -ForestMode {2} -DatabasePath {3} -SysvolPath {4} -LogPath {5} -NoDnsOnNetwork -DomainNetbiosName {6} -SafeModeAdministratorPassword `$Credential.Password", $DomainName, $DomainMode, $ForestMode, $DatabasePath, $SysvolPath, $LogPath, $NetBIOSName)
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	#NB the parameters on the following command line are parameters passed to this script
	$wf = Test-ADDSForestInstallation -DomainName $DomainName -InstallDNS -DomainMode $DomainMode -ForestMode $ForestMode -DatabasePath $DatabasePath -SysvolPath $SysvolPath -LogPath $LogPath -NoDnsOnNetwork -DomainNetbiosName $NetBIOSName -SafeModeAdministratorPassword $Credential.Password
	$message =  [string]::format("wf Status is {0}", $wf.Status)
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message

	return $wf
}

function Install-Forest
{
	PARAM( 
        [Parameter(Mandatory=$true)] [PSCredential]$Credential,
	    [Parameter(Mandatory=$true)] [string]$NetBIOSName, 
	    [Parameter(Mandatory=$false)] [boolean]$CreateDNSDelegation = $false,
	    [Parameter(Mandatory=$false)] [PSCredential]$DNSDelegationCredential = $null
    )
	# exit
    $message = [string]::Format("Calling Install-ADDSForest -DomainName {0} -InstallDNS -DomainMode {1} -ForestMode {2} -DatabasePath {3} -SysvolPath {4} -LogPath {5} -Force -NoDnsOnNetwork -DomainNetbiosName {6} -SafeModeAdministratorPassword `$Credential.Password -NoRebootOnCompletion", $DomainName, $DomainMode, $ForestMode, $DatabasePath, $SysvolPath, $LogPath, $NetBIOSName)
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    $wi = $null
    if($CreateDNSDelegation -eq $true)
    {
	    $wi = Install-ADDSForest -DomainName $DomainName -InstallDNS -DomainMode $DomainMode -ForestMode $ForestMode -DatabasePath $DatabasePath -SysvolPath $SysvolPath -LogPath $LogPath -Force -NoDnsOnNetwork -DomainNetbiosName $NetBIOSName -SafeModeAdministratorPassword $Credential.Password -NoRebootOnCompletion -CreateDNSDelegation -DNSDelegationCredential ${DNSDelegationCredential}
    }
    else
    {
	    $wi = Install-ADDSForest -DomainName $DomainName -InstallDNS -DomainMode $DomainMode -ForestMode $ForestMode -DatabasePath $DatabasePath -SysvolPath $SysvolPath -LogPath $LogPath -Force -NoDnsOnNetwork -DomainNetbiosName $NetBIOSName -SafeModeAdministratorPassword $Credential.Password -NoRebootOnCompletion
    }

	$message = [string]::format("Install-ADDSForest Status is {0}", $wi.Status)
    Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
	return $wi
}

Function Test-DNSDomainName
{
    PARAM( [string]$DNSDomainName )

    # complete FQDN cannot be more than 254 cahracters and not less than four characters
    # top-level domain is at least two characters
    # each naming component ("label" per RFCs) cannot exceed 64 characters
    # a label cannot start or end with hyphens (RFC 952)
    # a label can start with numbers (RFC 1123)
    # regex from "bkr" at http://stackoverflow.com/questions/11809631/fully-qualified-domain-name-validation
    # (?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}(?<!-)\.)+[a-zA-Z]{2,63}$)
    $bReturn = $false
    $regex = "(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}(?<!-)\.)+[a-zA-Z]{2,63}$)"
    if ($DNSDomainName -match $regex)
    {
        $bReturn = $true
    }
    return $bReturn

    # test cases
    #write-host these should succeed
    #"www.microsoft.com" -match $regex
    #"a.bc" -match $regex
    #"911.com" -match $regex
    #"a.b.c.d.e.f.g.h.i.j.k.l.es" -match $regex
    #"123456789012345678901234567890123456789012345678901234567890123.ch" -match $regex
    #write-host these should fail
    #"my_domain.com" -match $regex
    #"-domain.com" -match $regex
    #"domain-.com" -match $regex
    #"domain.a" -match $regex
    #"aa" -match $regex
    #"1234567890123456789012345678901234567890123456789012345678901234.ch" -match $regex
}

Function Test-PasswordComplexity1
{
    Param(
        [Parameter(Mandatory=$true)] [SecureString]$Passwd
    )
    $bReturn = $false;
    # requires retrieving secure-string from memory. Do not persist password in available. Work directly with BSTR.
    $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword)
    $ptPwd = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR)

    # $pwd = 'Pa$$w0rd'
    #password must be at least 15 characters
    $PasswordValidationTestsPassed = @()
    $PasswordValidationTestsFailed = @()

    $pwdLength = $ptPwd -match "(?=.{15,})"
    if ($pwdLength -eq $true)
    {
        $message = [string]::format("password is at least 15 characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password is less than 15 characters.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one lowercase character
    $pwdLowercase = $ptPwd -cmatch "[a-z]"
    if ($pwdLowercase -eq $true)
    {
        $message = [string]::format("password contains at least one lowercase characters.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one lowercase character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }

    #password must contain at least one UPPERCASE character
    $pwdUpperCase = $ptPwd -cmatch "[A-Z]"
    if ($pwdUpperCase -eq $true)
    {
        $message = [string]::format("password contains at least one UPPERCASE character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contain at least one UPPERCASE character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # password must contain at least one numeric or non-alphanumeric character
    $pwdNumericOrSpecial = $ptPwd -match "(?=.[\d\W])"
    if ($pwdNumericOrSpecial -eq $true)
    {
        $message = [string]::format("password contains at least one digit or special character.")
        $PasswordValidationTestsPassed += $message
        # write-host -fore green $message
    }
    else
    {
        $message = [string]::format("password does not contains at least one digit or special character.")
        $PasswordValidationTestsFailed += $message
        # write-host -fore red $message
    }
    # write summary
    if ($PasswordValidationTestsFailed.Length -gt 0)
    {
        write-host -fore red Password Failed Validation for the following reasons:
        for ( $n = 0; $n -lt $PasswordValidationTestsFailed.Length; $n++)
        {
            write-host -fore red $PasswordValidationTestsFailed[$n]
        }
    }
    else
    {
        write-host -fore green Password Passed Validation:
        for ( $n = 0; $n -lt $PasswordValidationTestsPassed.Length; $n++)
        {
            write-host -fore green $PasswordValidationTestsPassed[$n]
        }
        $bReturn = $true
    }
    return $bReturn
}

function Initialize-ADDSForest
{
    PARAM(
	[Parameter(Mandatory=$true)] [string]$DomainName,
	[Parameter(Mandatory=$false)] [string]$DatabasePath = "$env:SystemRoot\NTDS",
	[Parameter(Mandatory=$false)] [string]$SysvolPath = "$env:SystemRoot\SYSVOL",
	[Parameter(Mandatory=$false)] [string]$LogPath = "$env:SystemRoot\Logs",
	[Parameter(Mandatory=$false)] [string]$DomainMode = "Win2012R2",
	[Parameter(Mandatory=$false)] [string]$ForestMode = "Win2012R2",
	[Parameter(Mandatory=$false)] [boolean]$CreateDNSDelegation = $false,
	[Parameter(Mandatory=$false)] [PSCredential]$DNSDelegationCredential = $null
    )

    
    ################################
    # MAIN
    ################################
    # make sure user is an Administrator or we will fail later on
$currPrincipal = ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent())
If (-NOT ($currPrincipal.IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")))
{
    $message = [string]::Format("You are not logged on as an administrator. Try again with elevated permissions.")
    Write-Host -ForegroundColor Red $message
    Exit
}
    
    #$AppPath = 'C:\DeployPam'
    $AppPath = Convert-Path(Get-Location)
    # See if start location path is valid
    #if (-NOT (Test-Path $AppPath))
    #{
    #    $message = [string]::Format("Path {0} does not exist. Exiting...", $AppPath)
    #    Write-Host -ForegroundColor Red Path $AppPath does not exist. Exiting...
    #    Exit
    #}
    # set location
    $error.Clear()
    Try
    {
        # NB: Need to set ErrorAction to Stop to catch errors from Set-Location
        Set-Location $AppPath -ErrorAction Stop
    }
    Catch
    {
        $message = [string]::Format("Exception Category Reason: {0}. Exception Message: {1}", $_.CategoryInfo.Reason, $_.Exception.Message)
        Write-Host -ForegroundColor Red $message
        Exit
    }
    
    $error.Clear()
    $global:deployFolder = Convert-Path(Get-Location)
    $moduleName = 'CommonFunctions.psm1'
    $res = Import-CommonFunctions -deployFolder $deployFolder -moduleName $moduleName
    if ($res -eq $false)
    {
        $message = [string]::Format("Red Could not load module {0}. Exiting...", $moduleName)
        Write-Host -ForegroundColor $message    
        Exit
    }
    
    #$global:LogFileName = Get-LogFileName InstallForest "C:\"
    if([String]::IsNullOrEmpty($global:LogFileName))
    {
        $global:LogFileName = Get-LogFileName InstallForest "$env:SystemDrive\"
    }

    $global:EventLogName = 'Application'
    $global:EventLogSource = 'InstallForest'

    $res = Register-EventLogSource -eventLogName $EventLogName -eventLogSource $EventLogSource
    
    # while we have parameterized $DomainMode and $ForestMode, we do not want to recognize other values passed from command line
    #    so if a different value has been passed on the command line, restore the default values
    if (($DomainMode -ne "Win2012R2") -and ($DomainMode -ne "WinThreshold"))
    {
    	$DomainMode = "Win2012R2"
        $message = [string]::Format("Setting DomainMode to {0}.", $DomainMode)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
    }
    else
    {
        $message = [string]::Format("DomainMode will be {0}.", $DomainMode)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    }
    if (($ForestMode -ne "Win2012R2") -and ($ForestMode -ne "WinThreshold"))
    {
    	$ForestMode = "Win2012R2"
        $message = [string]::Format("Setting ForestMode to {0}.", $ForestMode)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
    }
    else
    {
        $message = [string]::Format("ForestMode will be {0}.", $ForestMode)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    }
    
    #
    # Validate Domain Name
    #
    $IsWindowsServer = Test-IsWindowsServer
    $IsVersionOK = Test-IsVersion2012R2orHigher
    $IsInDomain = Test-IsInDomain
    
    if (Test-DNSDomainName -DNSDomainName $DomainName)
    {
        $message = [string]::Format("Domain name {0} passed validation", $DomainName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    }
    else
    {
        $message = [string]::format("Domain name {0} is not a valid DNS domain name.", $DomainName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
        Exit
    }
    
    #
    # Get Directory Service Restore Mode Password
    #
    $DirSvcRestoreModeCred = Get-DirectoryServiceRestoreModePassword
    if ($DirSvcRestoreModeCred -ne $null)
    {
        $message = 'Successfully obtained Directory Service Restore Mode Password'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    }
    else
    {
        $message = 'Did not obtain Directory Service Restore Mode password'
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Warning -message $message
    	exit
    }
    #
    #	Get Domain Admin Credentials
    #
    # No need to get Domain Admin  credentials. The current local administrator will become the Domain Administrator
    #
    # Perform validation checks
    #
    $IsStaticIP = Test-IsStaticIP
    $IsDatabasePathValid = Test-IsDatabasePathValid
    $IsSysvolPathValid = Test-IsSysvolPathValid
    $IsLogPathValid = Test-IsLogPathValid
    $IsDomainSingleRoot = Test-IsDomainSingleRoot
    $GetNetBIOSName = Get-NetBIOSName $DomainName
    $IsADDomainServicesFeatureAvailable = Test-IsADDomainServicesFeatureAvailable
    $IsADDomainServicesFeatureInstallationRequired = Test-IsADDomainServicesFeatureInstallationRequired
    $IsDNSFeatureAvailable = Test-IsDNSFeatureAvailable
    $IsDNSFeatureInstallationRequired = Test-IsDNSFeatureInstallationRequired
    
    # create local users and groups
    $scriptName = 'Create-MIMPAMLocalUsers.ps1'
    if (Test-Path $AppPath\$scriptName)
    {
        .\Create-MIMPAMLocalUsers.ps1
    }
    if ($IsADDomainServicesFeatureAvailable -and $IsADDomainServicesFeatureInstallationRequired)
    {
    	$InstallADDomainServices = Install-ADDomainServices
    	if ($InstallADDomainServices -ne $true)
    	{
            $message = "AD-Domain-Services feature could not be installed"
    		#write-host -fore red AD-Domain-Services feature could not be installed
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
    		exit
    	}
    }
    
    if ($IsDNSFeatureAvailable -and $IsDNSFeatureInstallationRequired)
    {
    	$InstallDNS = Install-DNS
    	if ($InstallDNS -ne $true)
    	{
            $message = "DNS feature could not be installed"
    		# write-host -fore red DNS feature could not be installed
            Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
    		exit
    	}
    }
    
    if (($IsADDomainServicesFeatureInstallationRequired -eq $false) -or ($InstallADDomainServices -eq $true))
    {
        $message = [string]::Format("Calling Test-ForestCreation -Credential {0} -NetBIOSName {1}", $DirSvcRestoreModeCred, $GetNetBIOSName)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    	$tfc = Test-ForestCreation -Credential $DirSvcRestoreModeCred -NetBIOSName $GetNetBIOSName
        $message = [string]::format("Test Forest Creation Status is {0}", $tfc.Status)
    	# write-host Test Forest Creation Status is $tfc.Status
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    }
    
    if ($tfc.Status -eq "Success")
    {
    	# Test-ForestCreation succeeded so try to Install Forest
        $message = [string]::format("Installing forest...")
    	# write-host Installing Forest
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Information -message $message
    
        if($CreateDNSDelegation -eq $true)
        {
    	    $fi = Install-Forest -Credential $DirSvcRestoreModeCred -NetBIOSName $GetNetBIOSName -CreateDNSDelegation $true -DNSDelegationCredential $DNSDelegationCredential
        }
        else
        {
    	    $fi = Install-Forest -Credential $DirSvcRestoreModeCred -NetBIOSName $GetNetBIOSName
        }
    	# IMPORTANT: the computer will automatically restart when Install-ADDSForest completes successfully
    	# the following line may never be hit
        $message = [string]::format("Install Forest Creation Status is {0}", $fi.Status)
    	#write-host Install Forest Creation Status is $fi.Status
        Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Information -message $message
    }
    else
    {
        $message = [string]::format("Test Forest Creation failed with error {0}", $tfc)
        Write-LogsAndDisplay -EventID 11005 -ErrorLevel Error -message $message
        exit
    }
    $Restart = $false
    if ($fi.Status -ne "Success")
    {
        $message = [string]::format("Forest failed to install. Status is {0}", $fi.Status)
    	#write-host -fore red Forest failed to install. Status is $fi.Status
        Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Error -message $message
    
    }
    else
    {
        $Restart = $true
    }
    #DONE
    
    #Show Status information....
    write-host Is Windows Server? $IsWindowsServer
    write-host Is Windows Version? OK $IsVersionOK
    write-host Is Machine In Domain? $IsInDomain
    write-host Is There a Static IP Address? $IsStaticIP
    write-host Is the Path for NTDS Database Valid? $IsDatabasePathValid 
    write-host Is the Path for Sysvol Valid? $IsSysvolPathValid
    write-host Is the Path for Logs Valid? $IsLogPathValid
    write-host Is The Domain DNS a Single Root? $IsDomainSingleRoot
    write-host NetBIOSName is $GetNetBIOSName
    write-host Is AD-Domain-Services Feature Available? $IsADDomainServicesFeatureAvailable
    write-host Is Installation of AD-Domain-Services Feature Required? $IsADDomainServicesFeatureInstallationRequired
    write-host Is DNS Feature Available? $IsDNSFeatureAvailable
    write-host Was Installation of AD-Domain-Services Feature Successful? $InstallADDomainServices
    write-host Is Installation of DNS Feature Required? $IsDNSFeatureInstallationRequired
    write-host Was Installation of DNS Feature Successful? $InstallDNS
    write-host Result of Test-ADDSInstallation: $tfc.Status
    
    $message = "FINAL REPORT`n"
    $message = $message + "Is Windows Server? $IsWindowsServer`n"
    $message = $message + "Is Windows Version? OK $IsVersionOK`n"
    $message = $message + "Is Machine In Domain? $IsInDomain`n"
    $message = $message + "Is There a Static IP Address? $IsStaticIP`n"
    $message = $message + "Is the Path for NTDS Database Valid? $IsDatabasePathValid`n"
    $message = $message + "Is the Path for Sysvol Valid? $IsSysvolPathValid`n"
    $message = $message + "Is the Path for Logs Valid? $IsLogPathValid`n"
    $message = $message + "Is The Domain DNS a Single Root? $IsDomainSingleRoot`n"
    $message = $message + "Using NetBIOSName $GetNetBIOSName`n"
    $message = $message + "Is AD-Domain-Services Feature Available? $IsADDomainServicesFeatureAvailable`n"
    $message = $message + "Is Installation of AD-Domain-Services Feature Required? $IsADDomainServicesFeatureInstallationRequired`n"
    $message = $message + "Is DNS Feature Available? $IsDNSFeatureAvailable`n"
    $message = $message + "Was Installation of AD-Domain-Services Feature Successful? $InstallADDomainServices`n"
    $message = $message + "Is Installation of DNS Feature Required? $IsDNSFeatureInstallationRequired`n"
    $message = $message + "Was Installataion of DNS Feature Successful? $InstallDNS`n"
    $message = $message + "Result of Test-ADDSInstallation: $tfc"
    
    #write-host $message
    #Write-LogFile -logFileName $global:logFileName -ErrorLevel Information -message $message
    Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Information -message $message
    
    #####################################
    # Clean Up After Using
    #####################################
    
    # Reboot if required
    if ($Restart -eq $true)
    {
        $error.Clear()
        $message = 'Restart Required'
        #write-host Restart Required
        Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Information -message $message
    
        $delay = 10
        $message = [string]::Format("Restarting computer in {0} seconds", $delay)
        #Write-Host Shutting down in $delay seconds
        Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Information -message $message
        Try
        {
            $message = [string]::format("Calling shutdown /r /t {0}", $delay)
            Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Information -message $message
            # Cannot call Restart-Comptuer as the RPC server is not yet running
            shutdown /r /t $delay
        }
        Catch
        {
            if ($error[0] -ne $null)
            {
                $message = $error[0].Exception.Message
                Write-LogsAndDisplay  -EventID 11005 -ErrorLevel Error -message $message
            }
        }
    }
}


#Category   : OperationStopped
#Activity   : Restart-Computer
#Reason     : InvalidOperationException
#TargetName : CORPDC01
#TargetType : String


#Error Restarting computer...
#Restart-Computer : The computer 10 is skipped. Fail to retrieve its LastBootUpTime via the WMI service with the following error message: 
#The RPC server is unavailable. (Exception from HRESULT: 0x800706BA).
#At C:\MIMPAMInstall\initialize-ADDSForest.ps1:892 char:5
#+     Restart-Computer -Wait 10
#+     ~~~~~~~~~~~~~~~~~~~~~~~~~
#    + CategoryInfo          : OperationStopped: (10:String) [Restart-Computer], InvalidOperationException
#    + FullyQualifiedErrorId : RestartComputerSkipped,Microsoft.PowerShell.Commands.RestartComputerCommand

Export-ModuleMember -Function Initialize-ADDSForest
