#########################################################################################
# PAMValidation.psm1 
# Module used to set up and end to end validate
# basic PAM scenarios.
#
# Function List (to be run in this order)
# 1. Create-PAMValidationCorpDCConfig 
#		used to add users and Security groups.
#		Run On: CorpDC
#		Run as: Corp Domain admin
#		Clean Function: Remove-PAMValidationCorpDCConfig
#
# 2. Move-PamValidationUsersToPam 
#		Creates PAM users and groups and roles. Requires PAM trust to be set up and users/groups created in AD
#		Run On: PAM server
#		Run as: MIM admin
#		Clean Function: Remove-PamValidationUsersFromPam
#
# 3. Enable-PamUsersCorpClientRemote
#			Allows users created in this script and priv versions of those users to access remote psh/		
#		Run On: Corp Client
#		Run as: Priv version of pam requestor user: priv.PamRequestor
#
# 4. Test-PamValidationScenarioNoApprovalRequest
#		Runs through PAM scenario:
#			a. Validate not in the target group
#			b. create pam request
#			c. validate in the target group			
#		Run On: Corp Client
#		Run as: Priv version of pam requestor user: priv.PamRequestor
#		Clean Function: none needed
#
###############################################
# Exported Utility Functions
# 1. Get-PAMValidationConfig reads the PamConfig.json file which contains usernames and passwords for validation.
# 
#########################################################################################
#region Logging
#----------------------------------------------------------------
#####This came from CommonFunctions.psm1#######
Function Get-LogFileName
{
    # log file name returned in format [$path\]filename_20150603T120101.log
    param(
        [Parameter(Mandatory=$true)] [string]$fileName, 
        [Parameter(Mandatory=$false)] [string]$path)
    if (!$path.EndsWith("\"))
    {
        $path = $path + "\"
    }
    $d = (Get-Date).ToString("yyyyMMddThhmmss")
    return $path + $filename+ "_" + $d + ".log"
}

Function Import-PAMDeployModule
{
    Param(
    [string]$deployFolder,
    [string]$moduleName
    )
    $bReturn = $false
    $error.Clear()
    $modulePath = $deployFolder + '\' + $moduleName
    Write-Host -ForegroundColor Green Attempting to load module $modulePath
    if (Test-Path $modulePath)
    {
        Try
        {
            import-module $modulePath 
            Write-Host -ForegroundColor Green Successfully loaded module $modulePath
        $bReturn = $true
        }
        Catch
        {
            if ($error.Count -gt 0)
            {
                Write-Host -ForegroundColor Red $error[0].Exception.Message
            exit
            }
        }
    }
    else
    {
        $message = [string]::Format("Did not find {0} in {1}", $moduleName, $deployFolder)
        Write-host -ForegroundColor Red $message
        exit
    }
    return $bReturn
}
#####This came from CommonFunctions.psm1#######

#Other modules set it, but since this if loaded first, it needs to be initialized here.
$global:EventLogName = 'Application'
$global:EventLogSource = 'MIM-PAM Install'
$CurrentScriptLocation= Get-Location
$PSModulesLocation = $CurrentScriptLocation
$CommonFunctions = "CommonFunctions.psm1"

$installLogPath = "$ENV:AppData" + "\MIMPAMInstall"

if((Test-Path -Path $installLogPath) -eq $false)
{
    new-Item -ItemType directory -Path $installLogPath
}

Import-PAMDeployModule $PSModulesLocation $CommonFunctions
$global:LogFileName = Get-LogFileName MIMPAMInstall $installLogPath

$res = Register-EventLogSource -eventLogName $EventLogName -eventLogSource $EventLogSource

$ConfigFileName = "PAMDeploymentConfig.xml"
#ReadRequiredConfigInformation
$scriptPath = Convert-Path($CurrentScriptLocation)
$configFilePath = $scriptPath + "\" + $ConfigFileName
$Doc = New-Object System.Xml.XmlDocument
$Doc.Load($configfilePath)

$global:ADDSRole = $null
$global:CorpDomain = $null
$global:PrivDomain = $null

function Import-Settings
{
    param(
        [Parameter(Mandatory=$true)][Object]$xmlRoot
    )

    $domains = $xmlRoot.PAMDeploymentConfiguration.Domains
    $roles = $xmlRoot.PAMDeploymentConfiguration.Roles

    foreach($dom in $domains.Domain)
    {
        switch($dom.Type)
        {
            "CORP" { $Global:CorpDomain = $dom }
            "PRIV" { $Global:PrivDomain = $dom }
        }
    }

    foreach($role in $roles.Role)
    {
        switch($role.Type)
        {
            "ADDS" { $Global:ADDSRole = $role }
            "SQL" { $Global:SQLRole = $role }
            "SharePoint" { $Global:SPRole = $role }
            "MIM" { $Global:MIMRole = $role }
        }
    }
}

function Get-InnerValue
{
    param(
        [Parameter(Mandatory=$true)][Object]$rootDoc,
        [Parameter(Mandatory=$true)][string]$nodeName
    )
    $retVal = $rootDoc | select -ExpandProperty ChildNodes | 
                where {$_.Name -like $nodeName } | select -ExpandProperty InnerText 
    return $retVal
}

Import-Settings $Doc

function Get-DCOfType
{
    param(
         [Parameter(Mandatory=$true)][object]$rootObject,
         [Parameter(Mandatory=$true)][string]$role
    )

    $dcToReturn = $null

    foreach($dc in $rootObject.DCs.DC)
    {
        if($role.ToLowerInvariant().Equals($dc.Role.ToLowerInvariant()))
        {
            $dcToReturn = $dc
            break
        }
    }

    return $dcToReturn
}

function Get-RootObject
{
    param(
         [Parameter(Mandatory=$true)][string]$type
    ) 

    $rootObject = $null
    if($type.ToLowerInvariant().Equals("priv"))
    {
        $rootObject = $PrivDomain
    }
    elseif($type.ToLowerInvariant().Equals("corp"))
    {
        $rootObject = $CorpDomain
    }
    else
    {
        $message = "Invalid forest type while getting RootObject, exiting"
        Write-LogsAndDisplay -EventID 1 -ErrorLevel Warning -message $message
        throw $message
    }
    return $rootObject
}


#----------------------------------------------------------------
#endregion Logging
$script:config = $null
$script:configFilePath = ".\PamConfig.json"
$script:cachedCreds = $null


$infoEventId = 10
$warningEventId = 20
$errorEventId = 99
##############################################################
# Utility Functions
##############################################################
function Invoke-PamValidationFunction
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $functionDisplayName,

        [Parameter(Mandatory=$true)]
        [string] $functionName,
        
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )

	Get-Command -Name $functionName -Module $MyInvocation.MyCommand.Module.Name -ErrorAction SilentlyContinue -ErrorVariable getCommandError | Out-Null
	if ($getCommandError -ne $null)
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel 'Warning' ("In {0} cannot find {1}" -f $MyInvocation.MyCommand.Name, $functionName)
	}
	else
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel 'Information' "Beginning execution of $functionDisplayName"
		try 
		{
			& $functionName -configFilePath $configFilePath
			Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel 'Information' "Finished execution of $functionDisplayName"
		}
		catch [Exception] 
		{
			Write-LogsAndDisplay -EventID $errorEventId -ErrorLevel 'Error' `
				-message ("Execution of {0} failed with exception:`n{1}`nStack Trace:`n{2}" -f $functionDisplayName, $_.Exception.Message, $_.Exception.StackTrace)
		}
	}
}

function ConvertTo-PrivUserName ([string] $userName)
{
    return "priv.{0}" -f $userName
}

function Test-ADGroupContainsUser
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $GroupName,
        
        [Parameter(Mandatory=$true)]
        [string] $UserName
    )
    
    ipmo ActiveDirectory
    if (((Get-ADGroup -Identity $GroupName) -ne $null) -and `
        ($members = Get-ADGroupMember -Identity $GroupName).Count -gt 1)
    {
        foreach($groupMember in $members)
        { 
            if ($groupMember.name -eq $UserName) 
            { 
                return $true
            }
        }
    }
    return $false
}

function Test-LocalUserMemberOfGroup
{
	param
	(
		[Parameter(Mandatory=$true)]
		[string] $GroupDomain,

		[Parameter(Mandatory=$true)]
		[string] $GroupName
	)
	
	$targetGroupName = "{0}\{1}" -f $GroupDomain, $GroupName
	
	$allGroups = [System.Security.Principal.WindowsIdentity]::GetCurrent().Groups
	foreach ($Group in $allGroups) {
		$GroupSID = $Group.Value
		$GroupNameFromSID = New-Object System.Security.Principal.SecurityIdentifier($GroupSID)
		$GroupDisplayName = $GroupNameFromSID.Translate([System.Security.Principal.NTAccount])
		if ($GroupDisplayName -eq $targetGroupName)
		{
			return $true
		}
	}
	return $false
}

function Get-PamValidationCredential
{
	param
	(
		[Parameter(Mandatory=$true)]
        [string] $Purpose
	)
	
	Write-LogsAndDisplay -EventID 1 -ErrorLevel Information -message "Getting credentials for $purpose"
	$usn = Read-Host "Please enter user name  for $Purpose in <domain>\<username> format"
	$pwd = Read-Host "Enter password for $usn" -AsSecureString
	
	return (New-Object System.Management.Automation.PSCredential -ArgumentList @($usn, $pwd))
}
##############################################################
# Functions to read config.json file
##############################################################
function Get-PAMValidationConfigOld
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath
    )

    if ($script:config -eq $null)
    {
        $script:config = Read-PAMValidationConfig -configFilePath $configFilePath
    }
    $script:config.PrivOnly = [System.Convert]::ToBoolean($script:config.PrivOnly)
    return $script:config
}

function Get-PAMValidationConfig
{
    param
    (
        [Parameter(Mandatory=$false)]
		[string] $configFilePath

    )
    if ($Doc -eq $null)
    {
		$ConfigFileName = "PAMDeploymentConfig.xml"
		#ReadRequiredConfigInformation
		$scriptPath = Convert-Path($CurrentScriptLocation)
		$configFilePath = Join-Path -Path $scriptPath -ChildPath $ConfigFileName
		$Doc = New-Object System.Xml.XmlDocument
		$Doc.Load($configfilePath)

        Import-Settings $Doc
    }
	
	$config2 = New-Object PSObject
    $config2 | Add-Member NoteProperty -Name "PrivOnly" -Value $Doc.PAMDeploymentConfiguration.PrivOnly
    $config2.PrivOnly = [System.Convert]::ToBoolean($Doc.PAMDeploymentConfiguration.PrivOnly)

	$config2 | Add-Member NoteProperty -Name "CorpGroup" -Value $Doc.PAMDeploymentConfiguration.PamValidation.PamValidationGroup.Name
	
	$corpDom = New-Object PSObject
	$corpDomainName = Get-InnerValue $CorpDomain "NetbiosName"
	$corpDom | Add-Member NoteProperty -Name "DomainName" -Value $corpDomainName
	$corpDomainFqdn = Get-InnerValue $CorpDomain "DNSName"
	$corpDom | Add-Member NoteProperty -Name "DomainFqdn" -Value $corpDomainFqdn
	
	$corpDc = $CorpDomain.DCs.DC | Select-Object -First 1
	$corpDcName = $corpDc.Name
	$corpDom | Add-Member NoteProperty -Name "DCName" -Value $corpDcName
	$corpDcFqdn = "{0}.{1}" -f $corpDcName, $corpDomainFqdn
	$corpDom | Add-Member NoteProperty -Name "DCFqdn" -Value $corpDcFqdn

	$clientName = $Doc.PAMDeploymentConfiguration.PamValidation.PamValidationClient.Name
	$corpDom | Add-Member NoteProperty -Name "CorpClientMachineName" -Value $clientName
	$clientFqdn = "{0}.{1}" -f $clientName, $corpDomainFqdn
	$corpDom | Add-Member NoteProperty -Name "CorpClientFQDN" -Value $clientFqdn
	
	$config2 | Add-Member NoteProperty -Name "CorpDomain" -Value $corpDom
	
	#Priv domain
	$privDom = New-Object PSObject
	$PrivDomainName = Get-InnerValue $PrivDomain "NetbiosName"
	$privDom | Add-Member NoteProperty -Name "DomainName" -Value $PrivDomainName
	$PrivDomainFqdn = Get-InnerValue $PrivDomain "DNSName"
	$privDom | Add-Member NoteProperty -Name "DomainFqdn" -Value $PrivDomainFqdn
	
	$privDc = $PrivDomain.DCs.DC | Select-Object -First 1
	$privDcName = $privDc.Name
	$privDom | Add-Member NoteProperty -Name "DCName" -Value $privDcName
	$privDcFqdn = "{0}.{1}" -f $privDcName, $PrivDomainFqdn
	$privDom | Add-Member NoteProperty -Name "DCFqdn" -Value $privDcFqdn
	
	$config2 | Add-Member NoteProperty -Name "PrivDomain" -Value $privDom
	
	# PamUsers
	$pamUsersObject = New-Object PSObject
	$requestorUser = $doc.PAMDeploymentConfiguration.PamValidation.PamValidationUser | ? {$_.Type -eq "Requestor"}
	$pamUsersObject | Add-Member NoteProperty -Name "PamUserName" -Value $requestorUser.Name
	$pamUsersObject | Add-Member NoteProperty -Name "PamUserPwd" -Value $requestorUser.Password
		
	$approverUser = $doc.PAMDeploymentConfiguration.PamValidation.PamValidationUser | ? {$_.Type -eq "Approver"}
	$pamUsersObject | Add-Member NoteProperty -Name "PamApproverUsername" -Value $approverUser.Name
	$pamUsersObject | Add-Member NoteProperty -Name "PamApproverPwd" -Value $approverUser.Password
	
	$config2 | Add-Member NoteProperty -Name "PamUsers" -Value $pamUsersObject
	$script:config = $config2
    return $script:config
}

function Read-PAMValidationConfig
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath
    )
    if ([string]::IsNullOrEmpty($configFilePath) -or -not (Test-Path $configFilePath))
    {
        $configFilePath = $script:configFilePath
    }
    $pamConfigTxt = Get-Content $configFilePath -ErrorAction SilentlyContinue
    if ($pamConfigTxt -ne $null -and $pamConfigTxt.Count -ne 0)
    {
        return $pamConfigTxt -join "`n" | ConvertFrom-Json
    }
}

##############################################################
# Configuration functions to be called on Corp DC
##############################################################
function InternalCreate-PAMValidationCorpDCConfig
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath
    )

    $config = Get-PamValidationConfig -configFilePath $configFilePath
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message "This cmdlet will create 2 test users and a security group in the Corp domain."	

	ipmo ActiveDirectory
	
	
	$pamUser = $config.PamUsers.PamUserName
	if ((Get-ADUser -Filter "Name -eq '$pamUser'") -eq $null)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating user in AD with username: $pamUser"
		New-ADUser -SamAccountName $pamUser -Name $pamUser
		$pamUserPwd = ConvertTo-SecureString $config.PamUsers.PamUserPwd -AsPlainText -Force
		Set-ADAccountPassword -Identity $pamUser -NewPassword $pamUserPwd
		Set-ADUser -Identity $pamUser -Enabled $true -DisplayName $pamUser
	}
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "user already exists in AD with username: $pamUser"
	}
	
	$pamApprover = $config.PamUsers.PamApproverUsername
	if ((Get-ADUser -Filter "Name -eq '$pamApprover'") -eq $null)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating user in AD with username: $pamApprover"
		New-ADUser -SamAccountName $pamApprover -Name $pamApprover
		$pamApproverPwd = ConvertTo-SecureString $config.PamUsers.PamApproverPwd -AsPlainText -Force
		Set-ADAccountPassword -Identity $pamApprover -NewPassword $pamApproverPwd
		Set-ADUser -Identity $pamApprover -Enabled $true -DisplayName $pamApprover
	}
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "user already exists in AD with username: $pamApprover"
	}
	
	$pamGroup = $config.CorpGroup
	if ((Get-ADGroup -Filter "Name -eq '$pamGroup'") -eq $null)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating SG in AD with group name: $pamGroup"
		New-ADGroup -name $pamGroup -GroupCategory Security -GroupScope Global -SamAccountName $pamGroup
	}
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "SG already exists in AD with group name: $pamGroup"
	}
		
	if ((Get-ADGroupMember -Identity $pamGroup | ? { $_.Name -eq $pamApprover} | Measure-Object).Count -lt 1)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Adding user $pamApprover to SG: $pamGroup"
		Add-ADGroupMember -Identity $pamGroup -Members $pamApprover
	}
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "User $pamApprover is already a member of  SG: $pamGroup"
	}
	
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message "Finished creating test users on corp DC.`nNext step:`n`tLog into PAM server as MIM Admin user and run the Move-PamValidationUsersToPam cmdlet."
}

function Create-PAMValidationCorpDCConfig
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalCreate-PAMValidationCorpDCConfig" -configFilePath $configFilePath
}

function InternalRemove-PAMValidationCorpDCConfig
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath
    )
	
    $config = Get-PamValidationConfig -configFilePath $configFilePath
    ipmo ActiveDirectory

    $pamUser = $config.PamUsers.PamUserName
    if (($pamAdUser = Get-ADUser -Filter "Name -eq '$pamUser'") -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing user in AD with username: $pamUser"
        $pamAdUser | Remove-ADUser -Confirm:$false
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "User not found in AD with username: $pamUser"
	}
    
    $pamApprover = $config.PamUsers.PamApproverUsername
    if (($pamApproverAd = Get-ADUser -Filter "Name -eq '$pamApprover'") -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing user in AD with username: $pamApprover"
        $pamApproverAd | Remove-ADUser -Confirm:$false
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "User not found in AD with username: $pamApprover"
	}
    
    $pamGroup = $config.CorpGroup
    if (($pamAdGroup = Get-ADGroup -Filter "Name -eq '$pamGroup'") -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing SG in AD with group name: $pamGroup"
        $pamAdGroup | Remove-ADGroup -Confirm:$false
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "SG not found in AD with group name: $pamGroup"
	}
}

function Remove-PAMValidationCorpDCConfig
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalRemove-PAMValidationCorpDCConfig" -configFilePath $configFilePath
	
}

##############################################################
# Configuration functions to be called on PAM Server
##############################################################
function InternalMove-PamValidationUsersToPam
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("This cmdlet migrates PAM Validation test users to PAM.`nThis assumes that Create-PAMValidationCorpDCConfig has been run on the Corp DC.")	

	
    $config = Get-PamValidationConfig -configFilePath $configFilePath
    ipmo MIMPAM
    ipmo ActiveDirectory
    
    $corpCred = Get-PamValidationCredential -Purpose "Corp Domain Administrator"
    $privOnly = [System.Convert]::ToBoolean($config.PrivOnly)
    
	$groupName = $config.CorpGroup
    $pg = Get-PamGroup -SourceAccountName $groupName
    if ($pg -eq $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PAM group with group name: $groupName"
        $pg = New-PamGroup -SourceGroupName $groupName `
            -SourceDomain $config.CorpDomain.DomainFqdn `
            -SourceDC $config.CorpDomain.DCFqdn `
            -Credentials $corpCred `
            -PrivOnly:$privOnly
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "PAM group already exists with group name: $groupName"
	}
	
	$pamUserName = $config.PamUsers.PamUserName
    $pamUser = Get-PAMUser -SourceAccountName $pamUserName
    if ($pamUser -eq $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PAM user with username: $pamUserName"
        $pamUser = New-PAMUser -SourceDomain $config.CorpDomain.DomainFqdn `
            -SourceAccountName $pamUserName `
            -Credentials $corpCred `
            -PrivOnly:$privOnly
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "PAM user already exists with username: $pamUserName"
	}
    
	$pamApproverName = $config.PamUsers.PamApproverUsername
    $pamApprover = Get-PAMUser -SourceAccountName $pamApproverName
    if ($pamApprover -eq $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PAM user with username: $pamApproverName"
        $pamApprover = New-PAMUser -SourceDomain $config.CorpDomain.DomainFqdn `
            -SourceAccountName $pamApproverName `
            -Credentials $corpCred `
            -PrivOnly:$privOnly
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "PAM user already exists with username: $pamApproverName"		
	}
    if (-not $privonly)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Setting password for and enabling priv user with source username: $pamUserName"
        $pamUserPwd = ConvertTo-SecureString $config.PamUsers.PamUserPwd -AsPlainText -Force
        $pamUserPrivName = ConvertTo-PrivUserName $config.PamUsers.PamUserName
        Set-ADAccountPassword -Identity $pamUserPrivName -NewPassword $pamUserPwd
        Set-ADUser -Identity $pamUserPrivName -Enabled 1
       
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Setting password for and enabling priv user with source username: $pamApproverName"
        $pamApproverPwd = ConvertTo-SecureString $config.PamUsers.PamApproverPwd -AsPlainText -Force
        $pamApproverPrivName = ConvertTo-PrivUserName $config.PamUsers.PamApproverUsername
        Set-ADAccountPassword -Identity $pamApproverPrivName -NewPassword $pamApproverPwd
        Set-ADUser -Identity $pamApproverPrivName -Enabled 1
    }
    
    $pamRole = Get-PAMRole -DisplayName $config.CorpGroup
    if ($pamRole -eq $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PAM role with display name: $groupName"
        $pamRole = New-PAMRole -DisplayName $config.CorpGroup
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "PAM role already exists with display name: $groupName"				
	}
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("Configuring PAM role with display name: {0}`nPam Group Name: {1}`nCandidate DisplayName {2}`nTTL:{3}" -f $groupName, $groupName, $pamUserName, "00:05:00")
    Set-PamRole -Role $pamRole -Privileges $pg -Candidates $pamUser -TTL 00:05:00
	
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("Finished migrating PAM Validation test users to PAM.`nNext Step:`n`tLog onto the client machine and run the Enable-PamUsersCorpClientRemote cmdlet.")
}

function Move-PamValidationUsersToPam
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalMove-PamValidationUsersToPam" -configFilePath $configFilePath
}

function InternalRemove-PamValidationUsersFromPam
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )

    $config = Get-PamValidationConfig -configFilePath $configFilePath
    ipmo MimPAM
    
    $groupName = $config.CorpGroup
    if (($pg = Get-PamGroup -SourceAccountName $groupName) -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing PAM group with group name: $groupName"
        Remove-PamGroup -Group $pg -Force
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "Unable to find PAM group with group name: $groupName"		
	}
    
	$pamUserName = $config.PamUsers.PamUserName
    if (($pamUser = Get-PAMUser -SourceAccountName $pamUserName) -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing PAM user with user name: $pamUserName"
        Remove-PamUser -User $pamUser -Force
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "Unable to find PAM user with user name: $pamUserName"		
	}
    
	$pamApproverName = $config.PamUsers.PamApproverUsername
    if (($pamApprover = Get-PAMUser -SourceAccountName $pamApproverName) -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing PAM user with user name: $pamApproverName"
        Remove-PamUser -User $pamApprover -Force
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "Unable to find PAM user with user name: $pamApproverName"		
	}
    
	$pamRoleName = $config.CorpGroup
    if (($pamRole = Get-PAMRole -DisplayName $pamRoleName) -ne $null)
    {
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Removing PAM role with display name: $pamRoleName"
        Remove-PamRole -Role $pamRole -Force
    }
	else
	{
		Write-LogsAndDisplay -EventID $warningEventId -ErrorLevel Warning -message "Unable to find PAM role with display name: $pamRoleName"		
	}
}

function Remove-PamValidationUsersFromPam
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalRemove-PamValidationUsersFromPam" -configFilePath $configFilePath
}

##############################################################
# Validation Functions
##############################################################
function Add-UserToLocalAdministrators
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $MachineName,

        [Parameter(Mandatory=$true)]
        [string] $UserDomain,

        [Parameter(Mandatory=$true)]
        [string] $Username,

        [switch] $UseCachedCreds 		
    )
	
	Add-UserToLocalGroup -MachineName $MachineName -UserDomain $UserDomain -Username $Username -GroupName "Administrators" -UseCachedCreds:$UseCachedCreds
}

function Add-UserToRemoteDesktop
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $MachineName,

        [Parameter(Mandatory=$true)]
        [string] $UserDomain,

        [Parameter(Mandatory=$true)]
        [string] $Username,

        [switch] $UseCachedCreds 		
    )
	
	Add-UserToLocalGroup -MachineName $MachineName -UserDomain $UserDomain -Username $Username -GroupName "Remote Desktop Users" -UseCachedCreds:$UseCachedCreds
}

function Add-UserToRemoteManagement
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $MachineName,

        [Parameter(Mandatory=$true)]
        [string] $UserDomain,

        [Parameter(Mandatory=$true)]
        [string] $Username,

        [switch] $UseCachedCreds 		
    )
	
	Add-UserToLocalGroup -MachineName $MachineName -UserDomain $UserDomain -Username $Username -GroupName "Remote Management Users" -UseCachedCreds:$UseCachedCreds
}

function Add-UserToLocalGroup
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $MachineName,

        [Parameter(Mandatory=$true)]
        [string] $UserDomain,

        [Parameter(Mandatory=$true)]
        [string] $Username, 
		
        [Parameter(Mandatory=$true)]
        [string] $GroupName,
        
        [switch] $UseCachedCreds 		
    )
	
    if (-not $UseCachedCreds -or $script:cachedCreds -eq  $null)
    {
        $adminCred = Get-PamValidationCredential -Purpose "$MachineName Administrator"
        $script:cachedCreds = $adminCred    
    }
    else
    {
        $adminCred = $script:cachedCreds            
    }
    
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("Adding user {0}\{1} to local `"{2}`" group on {3} machine." -f $UserDomain, $Username, $GroupName, $MachineName)
    $machineSession = New-PSSession -ComputerName $MachineName -Credential $adminCred
    try {
    $results = Invoke-Command -Session $machineSession -ScriptBlock { `
        param([string]$domainName, [string] $username, [string] $groupName)
		$groupStr = "WinNT://./{0},group" -f $groupName
        $group = [ADSI]$groupStr;
        $containsUser = $false 
        $group.Members() | % { if (([ADSI]$_).InvokeGet("Name") -eq $username) { $containsUser = $true } }
        if (-not $containsUser)
        {
            $group.Add("WinNT://$domainName/$username")
        }
    } `
    -ArgumentList @($UserDomain, $Username, $GroupName)
}
finally
{
	if ($machineSession -ne $null)
{
	Remove-PsSession $machineSession
}
}
}

function Remove-UserFromLocalGroup
{
    param
    (
        [Parameter(Mandatory=$true)]
        [string] $MachineName,

        [Parameter(Mandatory=$true)]
        [string] $UserDomain,

        [Parameter(Mandatory=$true)]
        [string] $Username, 
		
        [Parameter(Mandatory=$true)]
        [string] $GroupName,
        
        [switch] $UseCachedCreds 		
    )
	
    if (-not $UseCachedCreds -or $script:cachedCreds -eq  $null)
    {
        $adminCred = Get-PamValidationCredential -Purpose "$MachineName Administrator"
        $script:cachedCreds = $adminCred    
    }
    else
    {
        $adminCred = $script:cachedCreds            
    }
    
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("Removing user {0}\{1} to local `"{2}`" group on {3} machine." -f $UserDomain, $Username, $GroupName, $MachineName)
    $machineSession = New-PSSession -ComputerName $MachineName -Credential $adminCred
	try 
	{
		$results = Invoke-Command -Session $machineSession -ScriptBlock { `
			param([string]$domainName, [string] $username, [string] $groupName)
			$groupStr = "WinNT://./{0},group" -f $groupName
			$group = [ADSI]$groupStr;
			$containsUser = $false 
			$group.Members() | % { if (([ADSI]$_).InvokeGet("Name") -eq $username) { $containsUser = $true } }
			if ($containsUser)
			{
				$group.Remove("WinNT://$domainName/$username")
			}
		} `
		-ArgumentList @($UserDomain, $Username, $GroupName)
	}
	finally 
	{
		if ($machineSession -ne $null)
		{
			Remove-PsSession $machineSession
		}
	}
}


function InternalEnable-PamUsersCorpClientRemote
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )

	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message "Enabling remote management and remote access for  test users on client machine.`nThis cmdlet requires Move-PamValidationUsersToPam to have been run on the PAM server."

    $config = Get-PamValidationConfig -configFilePath $configFilePath
    
    ## Add pamUser to corp client (for remote management)
	Add-UserToRemoteManagement -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamUserName -UseCachedCreds
    
    ## Add pamUser to corp client (for remote desktop)
    Add-UserToRemoteDesktop -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamUserName -UseCachedCreds
    
    ## Add pamApprover to corp client (for remote management)
    Add-UserToRemoteManagement -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamApproverUsername -UseCachedCreds
        
    ## Add pamApprover to corp client (for remote desktop)
    Add-UserToRemoteDesktop -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamApproverUsername -UseCachedCreds
        
    if (-not $config.PrivOnly)
    {
        $privPamUser = ConvertTo-PrivUserName $config.PamUsers.PamUserName
        $privPamApprover = ConvertTo-PrivUserName $config.PamUsers.PamApproverUsername
        
        ## Add priv.pamUser to corp client (for remote management)
        Add-UserToRemoteManagement -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamUser -UseCachedCreds
        
        ## Add priv.pamUser to corp client (for remote desktop)
        Add-UserToRemoteDesktop -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamUser -UseCachedCreds
        
        ## Add priv.pamApprover to corp client (for remote management)
        Add-UserToRemoteManagement -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamApprover -UseCachedCreds

		## Add priv.pamApprover to corp client (for remote desktop)
		Add-UserToRemoteDesktop -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamApprover -UseCachedCreds
    }
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message "Finished enabling remote access for  test users on client machine.`nNext Step:`n`tRun Test-PamValidationScenarioNoApprovalRequest as the priv.pamRequestor user on the corp client machine."
}

function InternalDisable-PamUsersCorpClientRemote
{
    param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
    
    $config = Get-PamValidationConfig -configFilePath $configFilePath
    
    ## Add pamUser to corp client (for remote management)
	Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamUserName -GroupName "Remote Management Users" -UseCachedCreds
    
    ## Add pamUser to corp client (for remote desktop)
    
    Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamUserName -GroupName "Remote Desktop USers" -UseCachedCreds
    
    ## Add pamApprover to corp client (for remote management)
	Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PAMApproverUserName -GroupName "Remote Management Users" -UseCachedCreds
        
    ## Add pamApprover to corp client (for remote desktop)
    Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
        -UserDomain $config.CorpDomain.DomainFqdn -Username $config.PamUsers.PamApproverUsername -GroupNAme "Remote Desktop Users" -UseCachedCreds
        
    if (-not $config.PrivOnly)
    {
        $privPamUser = ConvertTo-PrivUserName $config.PamUsers.PamUserName
        $privPamApprover = ConvertTo-PrivUserName $config.PamUsers.PamApproverUsername
        
        ## Add priv.pamUser to corp client (for remote management)
        
        Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamUser -GroupName "Remote Management Users" -UseCachedCreds
        
        ## Add priv.pamUser to corp client (for remote desktop)
        Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamUser -GroupName "Remote Desktop Users" -UseCachedCreds
        
        ## Add priv.pamApprover to corp client (for remote management)
        Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamApprover -GroupName "Remote Management Users" -UseCachedCreds

		## Add priv.pamApprover to corp client (for remote desktop)
		Remove-UserFromLocalGroup -MachineName $config.CorpDomain.CorpClientFQDN `
            -UserDomain $config.PrivDomain.DomainFqdn -Username $privPamApprover -GroupName "Remote Desktop Users" -UseCachedCreds
    }
}

function Disable-PamUsersCorpClientRemote
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalDisable-PamUsersCorpClientRemote" -configFilePath $configFilePath
}

function Enable-PamUsersCorpClientRemote
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalEnable-PamUsersCorpClientRemote" -configFilePath $configFilePath
	
}

function Test-PamValidationScenarioNoApprovalRequest
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath = ""
    )
	
	Invoke-PamValidationFunction -functionDisplayName $MyInvocation.MyCommand.Name -functionName "InternalTest-PamValidationScenarioNoApprovalRequest" -configFilePath $configFilePath

}

function InternalTest-PamValidationScenarioNoApprovalRequest
{
	param
    (
        [Parameter(Mandatory=$false)]
        [string] $configFilePath
    )


    $config = Get-PamValidationConfig -configFilePath $configFilePath
	
	
	
	$pamGroupName = $config.CorpGroup
	$pamRequestorUserName = ConvertTo-PrivUserName -userName $config.PamUsers.PamUserName
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
		-message ("The cmdlet Test-PamValidationScenarioNoApprovalRequest will validate that PAM is working as expected.`n" + `
			"It will take the following steps:`n" + `
			"`t1. Validate that the $pamRequestorUserName user is not a member of the $pamGroupName security group.`n" + `
			"`t2. Create the PAM Request for the $pamRequestorUserName user to gain access to the $pamGroupName security group.`n" + `
			"`t3. Validate that the $pamRequestorUserName user is a member of the $pamGroupName security group (equivalent to the PAM request going through).`n" + `
			"`t4. Wait until the request has expired. (Approximately 11 minutes)`n" + `
			"`t5. Validate that the $pamRequestorUserName user is no longer a member of the $pamGroupName security group.`n`n" + `
			"This cmdlet requires that Enable-PamUsersCorpClientRemote has been run.")
	
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Validating that current user is not a member of SG name: $pamGroupName"
	$preAccess = Test-LocalUserMemberOfGroup -GroupDomain $config.CorpDomain.DomainName -GroupName $pamGroupName
	if (-not $preAccess)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
			-message "************RESULT************`nCurrent user is not a member of SG name: $pamGroupName`n*********ENDRESULT************"
	}
	else
	{
		Write-LogsAndDisplay -EventID $errorEventId -ErrorLevel Error `
		-message "************RESULT************`nCurrent user is a member of SG name: $pamGroupName`n*********ENDRESULT************"
	}
	
	ipmo MimPAM
	$targetRole = Get-PAMRoleForRequest | ? { $_.DisplayName -eq $config.CorpGroup }
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PAM Request for role with display name: $pamGroupName"
	New-PamRequest -Role $targetRole
	
	Start-Sleep -Seconds 10 # let request go through
	
	$privRequestorName = "{0}\{1}" -f $config.PrivDomain.DomainName, (ConvertTo-PrivUserName -userName $config.PamUsers.PamUserName)
    $privRequestorPwd = ConvertTo-SecureString $config.PamUsers.PamUserPwd -AsPlainText -Force
    $privRequestorCred = New-Object System.Management.Automation.PSCredential -ArgumentList @($privRequestorName, $privRequestorPwd)
    
	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PSSession to validate new access"
    $machineSession = New-PSSession -ComputerName $config.CorpDomain.CorpClientMachineName -Credential $privRequestorCred
    if ($machineSession -ne $null)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Validating that current user is a member of SG name: $pamGroupName after request"
		$results = Invoke-Command -Session $machineSession -ScriptBlock { `
			param([string]$domainName, [string] $groupName)
			
			$targetGroupName = "{0}\{1}" -f $domainName, $groupName
	
			$allGroups = [System.Security.Principal.WindowsIdentity]::GetCurrent().Groups
			foreach ($Group in $allGroups) {
				$GroupSID = $Group.Value
				$GroupNameFromSID = New-Object System.Security.Principal.SecurityIdentifier($GroupSID)
				$GroupDisplayName = $GroupNameFromSID.Translate([System.Security.Principal.NTAccount])
				if ($GroupDisplayName -eq $targetGroupName)
				{
					return $true
				}
			}
			return $false
		} `
		-ArgumentList @($config.CorpDomain.DomainName, $config.CorpGroup)
		Remove-PsSession -Session $machineSession
		
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
			-message ("************RESULT************`nResults of Access check after Pam Request: {0}`n*********ENDRESULT************" -f $results)

		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
			-message ("Waiting 11 minutes for TTL to expire and PAM service to remove user from SGs")
		Start-Sleep -Seconds 660 # verify access denied after 5 minute window
	}
	else
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Unable to create remote pssession"
	}

	Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Creating PSSession to validate access expired"
	$machineSession2 = New-PSSession -ComputerName $config.CorpDomain.CorpClientMachineName -Credential $privRequestorCred
	if ($machineSession2 -ne $null)
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Validating that current user is not a member of SG name: $pamGroupName after request"
		$results2 = Invoke-Command -Session $machineSession2 -ScriptBlock { `
			param([string]$domainName, [string] $groupName)
			
			$targetGroupName = "{0}\{1}" -f $domainName, $groupName
	
			$allGroups = [System.Security.Principal.WindowsIdentity]::GetCurrent().Groups
			foreach ($Group in $allGroups) {
				$GroupSID = $Group.Value
				$GroupNameFromSID = New-Object System.Security.Principal.SecurityIdentifier($GroupSID)
				$GroupDisplayName = $GroupNameFromSID.Translate([System.Security.Principal.NTAccount])
				if ($GroupDisplayName -eq $targetGroupName)
				{
					return $true
				}
			}
			return $false
		} `
		-ArgumentList @($config.CorpDomain.DomainName, $config.CorpGroup)
		Remove-PsSession -Session $machineSession2

		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information `
			-message ("************RESULT************`nResults of Access check after Pam Request: {0}`n*********ENDRESULT************" -f $results2)
	}
	else
	{
		Write-LogsAndDisplay -EventID $infoEventId -ErrorLevel Information -message "Unable to create remote pssession"
	}
}

Export-ModuleMember -Function Get-PAMValidationConfig
Export-ModuleMember -Function Create-PAMValidationCorpDCConfig, Remove-PAMValidationCorpDCConfig
Export-ModuleMember -Function Create-PAMValidationAclResource, Remove-PAMValidationAclResource
Export-ModuleMember -Function Create-PAMValidationPamTrust
Export-ModuleMember -Function Move-PamValidationUsersToPam, Remove-PamValidationUsersFromPam
Export-ModuleMember -Function Enable-PamUsersCorpClientRemote
Export-ModuleMember -Function Disable-PamUsersCorpClientRemote
Export-ModuleMember -function Test-PamValidationScenarioNoApprovalRequest
Export-ModuleMember -function Test-LocalUserMemberOfGroup
Export-ModuleMember -Function Get-PAMValidationConfig2



##Utilty Functions
Export-ModuleMember -Function Test-ADGroupContainsUser
